import Security
from PyObjCTools.TestSupport import TestCase, min_os_level
import objc


class TestSecTask(TestCase):
    def test_types(self):
        self.assertIsCFType(Security.SecTaskRef)

    def test_functions(self):
        self.assertIsInstance(Security.SecTaskGetTypeID(), int)

        self.assertResultHasType(Security.SecTaskCreateWithAuditToken, objc._C_ID)
        self.assertResultIsCFRetained(Security.SecTaskCreateWithAuditToken)
        self.assertArgHasType(Security.SecTaskCreateWithAuditToken, 0, objc._C_ID)
        self.assertArgHasType(Security.SecTaskCreateWithAuditToken, 1, b"{?=[8I]}")

        self.assertResultHasType(Security.SecTaskCreateFromSelf, objc._C_ID)
        self.assertResultIsCFRetained(Security.SecTaskCreateFromSelf)
        self.assertArgHasType(Security.SecTaskCreateFromSelf, 0, objc._C_ID)

        self.assertResultHasType(Security.SecTaskCopyValueForEntitlement, objc._C_ID)
        self.assertResultIsCFRetained(Security.SecTaskCopyValueForEntitlement)
        self.assertArgHasType(Security.SecTaskCopyValueForEntitlement, 0, objc._C_ID)
        self.assertArgHasType(Security.SecTaskCopyValueForEntitlement, 1, objc._C_ID)
        self.assertArgHasType(
            Security.SecTaskCopyValueForEntitlement,
            2,
            objc._C_OUT + objc._C_PTR + objc._C_ID,
        )

        self.assertResultHasType(Security.SecTaskCopyValuesForEntitlements, objc._C_ID)
        self.assertResultIsCFRetained(Security.SecTaskCopyValuesForEntitlements)
        self.assertArgHasType(Security.SecTaskCopyValuesForEntitlements, 0, objc._C_ID)
        self.assertArgHasType(Security.SecTaskCopyValuesForEntitlements, 1, objc._C_ID)
        self.assertArgHasType(
            Security.SecTaskCopyValuesForEntitlements,
            2,
            objc._C_OUT + objc._C_PTR + objc._C_ID,
        )

    @min_os_level("10.12")
    def test_functions10_12(self):
        self.assertResultHasType(Security.SecTaskCopySigningIdentifier, objc._C_ID)
        self.assertResultIsCFRetained(Security.SecTaskCopySigningIdentifier)
        self.assertArgHasType(Security.SecTaskCopySigningIdentifier, 0, objc._C_ID)
        self.assertArgHasType(
            Security.SecTaskCopySigningIdentifier,
            1,
            objc._C_OUT + objc._C_PTR + objc._C_ID,
        )



import Security
from PyObjCTools.TestSupport import TestCase
import objc


class TestSecEncodeTransform(TestCase):
    def test_constants(self):
        self.assertIsInstance(Security.kSecBase64Encoding, str)
        self.assertIsInstance(Security.kSecBase32Encoding, str)
        self.assertIsInstance(Security.kSecZLibEncoding, str)
        self.assertIsInstance(Security.kSecEncodeTypeAttribute, str)
        self.assertIsInstance(Security.kSecLineLength64, str)
        self.assertIsInstance(Security.kSecLineLength76, str)
        self.assertIsInstance(Security.kSecEncodeLineLengthAttribute, str)
        self.assertIsInstance(Security.kSecCompressionRatio, str)

    def test_functions(self):
        self.assertResultHasType(Security.SecEncodeTransformCreate, objc._C_ID)
        self.assertResultIsCFRetained(Security.SecEncodeTransformCreate)
        self.assertArgHasType(Security.SecEncodeTransformCreate, 0, objc._C_ID)
        self.assertArgHasType(
            Security.SecEncodeTransformCreate, 1, objc._C_OUT + objc._C_PTR + objc._C_ID
        )



import Security
from PyObjCTools.TestSupport import TestCase


class TestMDS(TestCase):
    def test_unsuppported(self):
        self.assertFalse(hasattr(Security, "MDS_FUNCS"))
        self.assertFalse(hasattr(Security, "MDS_Initialize"))
        self.assertFalse(hasattr(Security, "MDS_Terminate"))
        self.assertFalse(hasattr(Security, "MDS_Install"))
        self.assertFalse(hasattr(Security, "MDS_Uninstall"))



import Security
from PyObjCTools.TestSupport import TestCase


class TestCipherSuite(TestCase):
    def test_constants(self):
        self.assertEqual(Security.SSL_NULL_WITH_NULL_NULL, 0x0000)
        self.assertEqual(Security.SSL_RSA_WITH_NULL_MD5, 0x0001)
        self.assertEqual(Security.SSL_RSA_WITH_NULL_SHA, 0x0002)
        self.assertEqual(Security.SSL_RSA_EXPORT_WITH_RC4_40_MD5, 0x0003)
        self.assertEqual(Security.SSL_RSA_WITH_RC4_128_MD5, 0x0004)
        self.assertEqual(Security.SSL_RSA_WITH_RC4_128_SHA, 0x0005)
        self.assertEqual(Security.SSL_RSA_EXPORT_WITH_RC2_CBC_40_MD5, 0x0006)
        self.assertEqual(Security.SSL_RSA_WITH_IDEA_CBC_SHA, 0x0007)
        self.assertEqual(Security.SSL_RSA_EXPORT_WITH_DES40_CBC_SHA, 0x0008)
        self.assertEqual(Security.SSL_RSA_WITH_DES_CBC_SHA, 0x0009)
        self.assertEqual(Security.SSL_RSA_WITH_3DES_EDE_CBC_SHA, 0x000A)
        self.assertEqual(Security.SSL_DH_DSS_EXPORT_WITH_DES40_CBC_SHA, 0x000B)
        self.assertEqual(Security.SSL_DH_DSS_WITH_DES_CBC_SHA, 0x000C)
        self.assertEqual(Security.SSL_DH_DSS_WITH_3DES_EDE_CBC_SHA, 0x000D)
        self.assertEqual(Security.SSL_DH_RSA_EXPORT_WITH_DES40_CBC_SHA, 0x000E)
        self.assertEqual(Security.SSL_DH_RSA_WITH_DES_CBC_SHA, 0x000F)
        self.assertEqual(Security.SSL_DH_RSA_WITH_3DES_EDE_CBC_SHA, 0x0010)
        self.assertEqual(Security.SSL_DHE_DSS_EXPORT_WITH_DES40_CBC_SHA, 0x0011)
        self.assertEqual(Security.SSL_DHE_DSS_WITH_DES_CBC_SHA, 0x0012)
        self.assertEqual(Security.SSL_DHE_DSS_WITH_3DES_EDE_CBC_SHA, 0x0013)
        self.assertEqual(Security.SSL_DHE_RSA_EXPORT_WITH_DES40_CBC_SHA, 0x0014)
        self.assertEqual(Security.SSL_DHE_RSA_WITH_DES_CBC_SHA, 0x0015)
        self.assertEqual(Security.SSL_DHE_RSA_WITH_3DES_EDE_CBC_SHA, 0x0016)
        self.assertEqual(Security.SSL_DH_anon_EXPORT_WITH_RC4_40_MD5, 0x0017)
        self.assertEqual(Security.SSL_DH_anon_WITH_RC4_128_MD5, 0x0018)
        self.assertEqual(Security.SSL_DH_anon_EXPORT_WITH_DES40_CBC_SHA, 0x0019)
        self.assertEqual(Security.SSL_DH_anon_WITH_DES_CBC_SHA, 0x001A)
        self.assertEqual(Security.SSL_DH_anon_WITH_3DES_EDE_CBC_SHA, 0x001B)
        self.assertEqual(Security.SSL_FORTEZZA_DMS_WITH_NULL_SHA, 0x001C)
        self.assertEqual(Security.SSL_FORTEZZA_DMS_WITH_FORTEZZA_CBC_SHA, 0x001D)
        self.assertEqual(Security.TLS_RSA_WITH_AES_128_CBC_SHA, 0x002F)
        self.assertEqual(Security.TLS_DH_DSS_WITH_AES_128_CBC_SHA, 0x0030)
        self.assertEqual(Security.TLS_DH_RSA_WITH_AES_128_CBC_SHA, 0x0031)
        self.assertEqual(Security.TLS_DHE_DSS_WITH_AES_128_CBC_SHA, 0x0032)
        self.assertEqual(Security.TLS_DHE_RSA_WITH_AES_128_CBC_SHA, 0x0033)
        self.assertEqual(Security.TLS_DH_anon_WITH_AES_128_CBC_SHA, 0x0034)
        self.assertEqual(Security.TLS_RSA_WITH_AES_256_CBC_SHA, 0x0035)
        self.assertEqual(Security.TLS_DH_DSS_WITH_AES_256_CBC_SHA, 0x0036)
        self.assertEqual(Security.TLS_DH_RSA_WITH_AES_256_CBC_SHA, 0x0037)
        self.assertEqual(Security.TLS_DHE_DSS_WITH_AES_256_CBC_SHA, 0x0038)
        self.assertEqual(Security.TLS_DHE_RSA_WITH_AES_256_CBC_SHA, 0x0039)
        self.assertEqual(Security.TLS_DH_anon_WITH_AES_256_CBC_SHA, 0x003A)
        self.assertEqual(Security.TLS_ECDH_ECDSA_WITH_NULL_SHA, 0xC001)
        self.assertEqual(Security.TLS_ECDH_ECDSA_WITH_RC4_128_SHA, 0xC002)
        self.assertEqual(Security.TLS_ECDH_ECDSA_WITH_3DES_EDE_CBC_SHA, 0xC003)
        self.assertEqual(Security.TLS_ECDH_ECDSA_WITH_AES_128_CBC_SHA, 0xC004)
        self.assertEqual(Security.TLS_ECDH_ECDSA_WITH_AES_256_CBC_SHA, 0xC005)
        self.assertEqual(Security.TLS_ECDHE_ECDSA_WITH_NULL_SHA, 0xC006)
        self.assertEqual(Security.TLS_ECDHE_ECDSA_WITH_RC4_128_SHA, 0xC007)
        self.assertEqual(Security.TLS_ECDHE_ECDSA_WITH_3DES_EDE_CBC_SHA, 0xC008)
        self.assertEqual(Security.TLS_ECDHE_ECDSA_WITH_AES_128_CBC_SHA, 0xC009)
        self.assertEqual(Security.TLS_ECDHE_ECDSA_WITH_AES_256_CBC_SHA, 0xC00A)
        self.assertEqual(Security.TLS_ECDH_RSA_WITH_NULL_SHA, 0xC00B)
        self.assertEqual(Security.TLS_ECDH_RSA_WITH_RC4_128_SHA, 0xC00C)
        self.assertEqual(Security.TLS_ECDH_RSA_WITH_3DES_EDE_CBC_SHA, 0xC00D)
        self.assertEqual(Security.TLS_ECDH_RSA_WITH_AES_128_CBC_SHA, 0xC00E)
        self.assertEqual(Security.TLS_ECDH_RSA_WITH_AES_256_CBC_SHA, 0xC00F)
        self.assertEqual(Security.TLS_ECDHE_RSA_WITH_NULL_SHA, 0xC010)
        self.assertEqual(Security.TLS_ECDHE_RSA_WITH_RC4_128_SHA, 0xC011)
        self.assertEqual(Security.TLS_ECDHE_RSA_WITH_3DES_EDE_CBC_SHA, 0xC012)
        self.assertEqual(Security.TLS_ECDHE_RSA_WITH_AES_128_CBC_SHA, 0xC013)
        self.assertEqual(Security.TLS_ECDHE_RSA_WITH_AES_256_CBC_SHA, 0xC014)
        self.assertEqual(Security.TLS_ECDH_anon_WITH_NULL_SHA, 0xC015)
        self.assertEqual(Security.TLS_ECDH_anon_WITH_RC4_128_SHA, 0xC016)
        self.assertEqual(Security.TLS_ECDH_anon_WITH_3DES_EDE_CBC_SHA, 0xC017)
        self.assertEqual(Security.TLS_ECDH_anon_WITH_AES_128_CBC_SHA, 0xC018)
        self.assertEqual(Security.TLS_ECDH_anon_WITH_AES_256_CBC_SHA, 0xC019)
        self.assertEqual(Security.TLS_ECDHE_PSK_WITH_AES_128_CBC_SHA, 0xC035)
        self.assertEqual(Security.TLS_ECDHE_PSK_WITH_AES_256_CBC_SHA, 0xC036)
        self.assertEqual(Security.TLS_PSK_WITH_CHACHA20_POLY1305_SHA256, 0xCCAB)

        self.assertEqual(Security.TLS_NULL_WITH_NULL_NULL, 0x0000)
        self.assertEqual(Security.TLS_RSA_WITH_NULL_MD5, 0x0001)
        self.assertEqual(Security.TLS_RSA_WITH_NULL_SHA, 0x0002)
        self.assertEqual(Security.TLS_RSA_WITH_RC4_128_MD5, 0x0004)
        self.assertEqual(Security.TLS_RSA_WITH_RC4_128_SHA, 0x0005)
        self.assertEqual(Security.TLS_RSA_WITH_3DES_EDE_CBC_SHA, 0x000A)
        self.assertEqual(Security.TLS_RSA_WITH_NULL_SHA256, 0x003B)
        self.assertEqual(Security.TLS_RSA_WITH_AES_128_CBC_SHA256, 0x003C)
        self.assertEqual(Security.TLS_RSA_WITH_AES_256_CBC_SHA256, 0x003D)
        self.assertEqual(Security.TLS_DH_DSS_WITH_3DES_EDE_CBC_SHA, 0x000D)
        self.assertEqual(Security.TLS_DH_RSA_WITH_3DES_EDE_CBC_SHA, 0x0010)
        self.assertEqual(Security.TLS_DHE_DSS_WITH_3DES_EDE_CBC_SHA, 0x0013)
        self.assertEqual(Security.TLS_DHE_RSA_WITH_3DES_EDE_CBC_SHA, 0x0016)
        self.assertEqual(Security.TLS_DH_DSS_WITH_AES_128_CBC_SHA256, 0x003E)
        self.assertEqual(Security.TLS_DH_RSA_WITH_AES_128_CBC_SHA256, 0x003F)
        self.assertEqual(Security.TLS_DHE_DSS_WITH_AES_128_CBC_SHA256, 0x0040)
        self.assertEqual(Security.TLS_DHE_RSA_WITH_AES_128_CBC_SHA256, 0x0067)
        self.assertEqual(Security.TLS_DH_DSS_WITH_AES_256_CBC_SHA256, 0x0068)
        self.assertEqual(Security.TLS_DH_RSA_WITH_AES_256_CBC_SHA256, 0x0069)
        self.assertEqual(Security.TLS_DHE_DSS_WITH_AES_256_CBC_SHA256, 0x006A)
        self.assertEqual(Security.TLS_DHE_RSA_WITH_AES_256_CBC_SHA256, 0x006B)
        self.assertEqual(Security.TLS_DH_anon_WITH_RC4_128_MD5, 0x0018)
        self.assertEqual(Security.TLS_DH_anon_WITH_3DES_EDE_CBC_SHA, 0x001B)
        self.assertEqual(Security.TLS_DH_anon_WITH_AES_128_CBC_SHA256, 0x006C)
        self.assertEqual(Security.TLS_DH_anon_WITH_AES_256_CBC_SHA256, 0x006D)
        self.assertEqual(Security.TLS_PSK_WITH_RC4_128_SHA, 0x008A)
        self.assertEqual(Security.TLS_PSK_WITH_3DES_EDE_CBC_SHA, 0x008B)
        self.assertEqual(Security.TLS_PSK_WITH_AES_128_CBC_SHA, 0x008C)
        self.assertEqual(Security.TLS_PSK_WITH_AES_256_CBC_SHA, 0x008D)
        self.assertEqual(Security.TLS_DHE_PSK_WITH_RC4_128_SHA, 0x008E)
        self.assertEqual(Security.TLS_DHE_PSK_WITH_3DES_EDE_CBC_SHA, 0x008F)
        self.assertEqual(Security.TLS_DHE_PSK_WITH_AES_128_CBC_SHA, 0x0090)
        self.assertEqual(Security.TLS_DHE_PSK_WITH_AES_256_CBC_SHA, 0x0091)
        self.assertEqual(Security.TLS_RSA_PSK_WITH_RC4_128_SHA, 0x0092)
        self.assertEqual(Security.TLS_RSA_PSK_WITH_3DES_EDE_CBC_SHA, 0x0093)
        self.assertEqual(Security.TLS_RSA_PSK_WITH_AES_128_CBC_SHA, 0x0094)
        self.assertEqual(Security.TLS_RSA_PSK_WITH_AES_256_CBC_SHA, 0x0095)
        self.assertEqual(Security.TLS_PSK_WITH_NULL_SHA, 0x002C)
        self.assertEqual(Security.TLS_DHE_PSK_WITH_NULL_SHA, 0x002D)
        self.assertEqual(Security.TLS_RSA_PSK_WITH_NULL_SHA, 0x002E)
        self.assertEqual(Security.TLS_RSA_WITH_AES_128_GCM_SHA256, 0x009C)
        self.assertEqual(Security.TLS_RSA_WITH_AES_256_GCM_SHA384, 0x009D)
        self.assertEqual(Security.TLS_DHE_RSA_WITH_AES_128_GCM_SHA256, 0x009E)
        self.assertEqual(Security.TLS_DHE_RSA_WITH_AES_256_GCM_SHA384, 0x009F)
        self.assertEqual(Security.TLS_DH_RSA_WITH_AES_128_GCM_SHA256, 0x00A0)
        self.assertEqual(Security.TLS_DH_RSA_WITH_AES_256_GCM_SHA384, 0x00A1)
        self.assertEqual(Security.TLS_DHE_DSS_WITH_AES_128_GCM_SHA256, 0x00A2)
        self.assertEqual(Security.TLS_DHE_DSS_WITH_AES_256_GCM_SHA384, 0x00A3)
        self.assertEqual(Security.TLS_DH_DSS_WITH_AES_128_GCM_SHA256, 0x00A4)
        self.assertEqual(Security.TLS_DH_DSS_WITH_AES_256_GCM_SHA384, 0x00A5)
        self.assertEqual(Security.TLS_DH_anon_WITH_AES_128_GCM_SHA256, 0x00A6)
        self.assertEqual(Security.TLS_DH_anon_WITH_AES_256_GCM_SHA384, 0x00A7)
        self.assertEqual(Security.TLS_PSK_WITH_AES_128_GCM_SHA256, 0x00A8)
        self.assertEqual(Security.TLS_PSK_WITH_AES_256_GCM_SHA384, 0x00A9)
        self.assertEqual(Security.TLS_DHE_PSK_WITH_AES_128_GCM_SHA256, 0x00AA)
        self.assertEqual(Security.TLS_DHE_PSK_WITH_AES_256_GCM_SHA384, 0x00AB)
        self.assertEqual(Security.TLS_RSA_PSK_WITH_AES_128_GCM_SHA256, 0x00AC)
        self.assertEqual(Security.TLS_RSA_PSK_WITH_AES_256_GCM_SHA384, 0x00AD)
        self.assertEqual(Security.TLS_PSK_WITH_AES_128_CBC_SHA256, 0x00AE)
        self.assertEqual(Security.TLS_PSK_WITH_AES_256_CBC_SHA384, 0x00AF)
        self.assertEqual(Security.TLS_PSK_WITH_NULL_SHA256, 0x00B0)
        self.assertEqual(Security.TLS_PSK_WITH_NULL_SHA384, 0x00B1)
        self.assertEqual(Security.TLS_DHE_PSK_WITH_AES_128_CBC_SHA256, 0x00B2)
        self.assertEqual(Security.TLS_DHE_PSK_WITH_AES_256_CBC_SHA384, 0x00B3)
        self.assertEqual(Security.TLS_DHE_PSK_WITH_NULL_SHA256, 0x00B4)
        self.assertEqual(Security.TLS_DHE_PSK_WITH_NULL_SHA384, 0x00B5)
        self.assertEqual(Security.TLS_RSA_PSK_WITH_AES_128_CBC_SHA256, 0x00B6)
        self.assertEqual(Security.TLS_RSA_PSK_WITH_AES_256_CBC_SHA384, 0x00B7)
        self.assertEqual(Security.TLS_RSA_PSK_WITH_NULL_SHA256, 0x00B8)
        self.assertEqual(Security.TLS_RSA_PSK_WITH_NULL_SHA384, 0x00B9)
        self.assertEqual(Security.TLS_AES_128_GCM_SHA256, 0x1301)
        self.assertEqual(Security.TLS_AES_256_GCM_SHA384, 0x1302)
        self.assertEqual(Security.TLS_CHACHA20_POLY1305_SHA256, 0x1303)
        self.assertEqual(Security.TLS_AES_128_CCM_SHA256, 0x1304)
        self.assertEqual(Security.TLS_AES_128_CCM_8_SHA256, 0x1305)
        self.assertEqual(Security.TLS_ECDHE_ECDSA_WITH_AES_128_CBC_SHA256, 0xC023)
        self.assertEqual(Security.TLS_ECDHE_ECDSA_WITH_AES_256_CBC_SHA384, 0xC024)
        self.assertEqual(Security.TLS_ECDH_ECDSA_WITH_AES_128_CBC_SHA256, 0xC025)
        self.assertEqual(Security.TLS_ECDH_ECDSA_WITH_AES_256_CBC_SHA384, 0xC026)
        self.assertEqual(Security.TLS_ECDHE_RSA_WITH_AES_128_CBC_SHA256, 0xC027)
        self.assertEqual(Security.TLS_ECDHE_RSA_WITH_AES_256_CBC_SHA384, 0xC028)
        self.assertEqual(Security.TLS_ECDH_RSA_WITH_AES_128_CBC_SHA256, 0xC029)
        self.assertEqual(Security.TLS_ECDH_RSA_WITH_AES_256_CBC_SHA384, 0xC02A)
        self.assertEqual(Security.TLS_ECDHE_ECDSA_WITH_AES_128_GCM_SHA256, 0xC02B)
        self.assertEqual(Security.TLS_ECDHE_ECDSA_WITH_AES_256_GCM_SHA384, 0xC02C)
        self.assertEqual(Security.TLS_ECDH_ECDSA_WITH_AES_128_GCM_SHA256, 0xC02D)
        self.assertEqual(Security.TLS_ECDH_ECDSA_WITH_AES_256_GCM_SHA384, 0xC02E)
        self.assertEqual(Security.TLS_ECDHE_RSA_WITH_AES_128_GCM_SHA256, 0xC02F)
        self.assertEqual(Security.TLS_ECDHE_RSA_WITH_AES_256_GCM_SHA384, 0xC030)
        self.assertEqual(Security.TLS_ECDH_RSA_WITH_AES_128_GCM_SHA256, 0xC031)
        self.assertEqual(Security.TLS_ECDH_RSA_WITH_AES_256_GCM_SHA384, 0xC032)
        self.assertEqual(Security.TLS_ECDHE_RSA_WITH_CHACHA20_POLY1305_SHA256, 0xCCA8)
        self.assertEqual(Security.TLS_ECDHE_ECDSA_WITH_CHACHA20_POLY1305_SHA256, 0xCCA9)
        self.assertEqual(Security.TLS_EMPTY_RENEGOTIATION_INFO_SCSV, 0x00FF)
        self.assertEqual(Security.SSL_RSA_WITH_RC2_CBC_MD5, 0xFF80)
        self.assertEqual(Security.SSL_RSA_WITH_IDEA_CBC_MD5, 0xFF81)
        self.assertEqual(Security.SSL_RSA_WITH_DES_CBC_MD5, 0xFF82)
        self.assertEqual(Security.SSL_RSA_WITH_3DES_EDE_CBC_MD5, 0xFF83)
        self.assertEqual(Security.SSL_NO_SUCH_CIPHERSUITE, 0xFFFF)

        self.assertEqual(Security.kSSLCiphersuiteGroupDefault, 0)
        self.assertEqual(Security.kSSLCiphersuiteGroupCompatibility, 1)
        self.assertEqual(Security.kSSLCiphersuiteGroupLegacy, 2)
        self.assertEqual(Security.kSSLCiphersuiteGroupATS, 3)
        self.assertEqual(Security.kSSLCiphersuiteGroupATSCompatibility, 4)



import Security
from PyObjCTools.TestSupport import TestCase


class Testcssmerr(TestCase):
    def test_unsuppported(self):
        self.assertFalse(hasattr(Security, "CSSM_BASE_ERROR"))
        self.assertFalse(hasattr(Security, "CSSM_ERRORCODE_MODULE_EXTENT"))
        self.assertFalse(hasattr(Security, "CSSM_ERRORCODE_CUSTOM_OFFSET"))
        self.assertFalse(hasattr(Security, "CSSM_ERRORCODE_COMMON_EXTENT"))
        self.assertFalse(hasattr(Security, "CSSM_ERRCODE"))
        self.assertFalse(hasattr(Security, "CSSM_ERRBASE"))
        self.assertFalse(hasattr(Security, "CSSM_ERR_IS_CONVERTIBLE"))
        self.assertFalse(hasattr(Security, "CSSM_ERR_TAG"))
        self.assertFalse(hasattr(Security, "CSSM_CSSM_BASE_ERROR"))
        self.assertFalse(hasattr(Security, "CSSM_CSSM_PRIVATE_ERROR"))
        self.assertFalse(hasattr(Security, "CSSM_CSP_BASE_ERROR"))
        self.assertFalse(hasattr(Security, "CSSM_CSP_PRIVATE_ERROR"))
        self.assertFalse(hasattr(Security, "CSSM_DL_BASE_ERROR"))
        self.assertFalse(hasattr(Security, "CSSM_DL_PRIVATE_ERROR"))
        self.assertFalse(hasattr(Security, "CSSM_CL_BASE_ERROR"))
        self.assertFalse(hasattr(Security, "CSSM_CL_PRIVATE_ERROR"))
        self.assertFalse(hasattr(Security, "CSSM_TP_BASE_ERROR"))
        self.assertFalse(hasattr(Security, "CSSM_TP_PRIVATE_ERROR"))
        self.assertFalse(hasattr(Security, "CSSM_KR_BASE_ERROR"))
        self.assertFalse(hasattr(Security, "CSSM_KR_PRIVATE_ERROR"))
        self.assertFalse(hasattr(Security, "CSSM_AC_BASE_ERROR"))
        self.assertFalse(hasattr(Security, "CSSM_AC_PRIVATE_ERROR"))
        self.assertFalse(hasattr(Security, "CSSM_MDS_BASE_ERROR"))
        self.assertFalse(hasattr(Security, "CSSM_MDS_PRIVATE_ERROR"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSSM_INVALID_ADDIN_HANDLE"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSSM_NOT_INITIALIZED"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSSM_INVALID_HANDLE_USAGE"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSSM_PVC_REFERENT_NOT_FOUND"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSSM_FUNCTION_INTEGRITY_FAIL"))
        self.assertFalse(hasattr(Security, "CSSM_ERRCODE_INTERNAL_ERROR"))
        self.assertFalse(hasattr(Security, "CSSM_ERRCODE_MEMORY_ERROR"))
        self.assertFalse(hasattr(Security, "CSSM_ERRCODE_MDS_ERROR"))
        self.assertFalse(hasattr(Security, "CSSM_ERRCODE_INVALID_POINTER"))
        self.assertFalse(hasattr(Security, "CSSM_ERRCODE_INVALID_INPUT_POINTER"))
        self.assertFalse(hasattr(Security, "CSSM_ERRCODE_INVALID_OUTPUT_POINTER"))
        self.assertFalse(hasattr(Security, "CSSM_ERRCODE_FUNCTION_NOT_IMPLEMENTED"))
        self.assertFalse(hasattr(Security, "CSSM_ERRCODE_SELF_CHECK_FAILED"))
        self.assertFalse(hasattr(Security, "CSSM_ERRCODE_OS_ACCESS_DENIED"))
        self.assertFalse(hasattr(Security, "CSSM_ERRCODE_FUNCTION_FAILED"))
        self.assertFalse(
            hasattr(Security, "CSSM_ERRCODE_MODULE_MANIFEST_VERIFY_FAILED")
        )
        self.assertFalse(hasattr(Security, "CSSM_ERRCODE_INVALID_GUID"))
        self.assertFalse(hasattr(Security, "CSSM_ERRCODE_OPERATION_AUTH_DENIED"))
        self.assertFalse(hasattr(Security, "CSSM_ERRCODE_OBJECT_USE_AUTH_DENIED"))
        self.assertFalse(hasattr(Security, "CSSM_ERRCODE_OBJECT_MANIP_AUTH_DENIED"))
        self.assertFalse(hasattr(Security, "CSSM_ERRCODE_OBJECT_ACL_NOT_SUPPORTED"))
        self.assertFalse(hasattr(Security, "CSSM_ERRCODE_OBJECT_ACL_REQUIRED"))
        self.assertFalse(hasattr(Security, "CSSM_ERRCODE_INVALID_ACCESS_CREDENTIALS"))
        self.assertFalse(hasattr(Security, "CSSM_ERRCODE_INVALID_ACL_BASE_CERTS"))
        self.assertFalse(hasattr(Security, "CSSM_ERRCODE_ACL_BASE_CERTS_NOT_SUPPORTED"))
        self.assertFalse(hasattr(Security, "CSSM_ERRCODE_INVALID_SAMPLE_VALUE"))
        self.assertFalse(hasattr(Security, "CSSM_ERRCODE_SAMPLE_VALUE_NOT_SUPPORTED"))
        self.assertFalse(hasattr(Security, "CSSM_ERRCODE_INVALID_ACL_SUBJECT_VALUE"))
        self.assertFalse(
            hasattr(Security, "CSSM_ERRCODE_ACL_SUBJECT_TYPE_NOT_SUPPORTED")
        )
        self.assertFalse(
            hasattr(Security, "CSSM_ERRCODE_INVALID_ACL_CHALLENGE_CALLBACK")
        )
        self.assertFalse(
            hasattr(Security, "CSSM_ERRCODE_ACL_CHALLENGE_CALLBACK_FAILED")
        )
        self.assertFalse(hasattr(Security, "CSSM_ERRCODE_INVALID_ACL_ENTRY_TAG"))
        self.assertFalse(hasattr(Security, "CSSM_ERRCODE_ACL_ENTRY_TAG_NOT_FOUND"))
        self.assertFalse(hasattr(Security, "CSSM_ERRCODE_INVALID_ACL_EDIT_MODE"))
        self.assertFalse(hasattr(Security, "CSSM_ERRCODE_ACL_CHANGE_FAILED"))
        self.assertFalse(hasattr(Security, "CSSM_ERRCODE_INVALID_NEW_ACL_ENTRY"))
        self.assertFalse(hasattr(Security, "CSSM_ERRCODE_INVALID_NEW_ACL_OWNER"))
        self.assertFalse(hasattr(Security, "CSSM_ERRCODE_ACL_DELETE_FAILED"))
        self.assertFalse(hasattr(Security, "CSSM_ERRCODE_ACL_REPLACE_FAILED"))
        self.assertFalse(hasattr(Security, "CSSM_ERRCODE_ACL_ADD_FAILED"))
        self.assertFalse(hasattr(Security, "CSSM_ERRCODE_INVALID_CONTEXT_HANDLE"))
        self.assertFalse(hasattr(Security, "CSSM_ERRCODE_INCOMPATIBLE_VERSION"))
        self.assertFalse(hasattr(Security, "CSSM_ERRCODE_INVALID_CERTGROUP_POINTER"))
        self.assertFalse(hasattr(Security, "CSSM_ERRCODE_INVALID_CERT_POINTER"))
        self.assertFalse(hasattr(Security, "CSSM_ERRCODE_INVALID_CRL_POINTER"))
        self.assertFalse(hasattr(Security, "CSSM_ERRCODE_INVALID_FIELD_POINTER"))
        self.assertFalse(hasattr(Security, "CSSM_ERRCODE_INVALID_DATA"))
        self.assertFalse(hasattr(Security, "CSSM_ERRCODE_CRL_ALREADY_SIGNED"))
        self.assertFalse(hasattr(Security, "CSSM_ERRCODE_INVALID_NUMBER_OF_FIELDS"))
        self.assertFalse(hasattr(Security, "CSSM_ERRCODE_VERIFICATION_FAILURE"))
        self.assertFalse(hasattr(Security, "CSSM_ERRCODE_INVALID_DB_HANDLE"))
        self.assertFalse(hasattr(Security, "CSSM_ERRCODE_PRIVILEGE_NOT_GRANTED"))
        self.assertFalse(hasattr(Security, "CSSM_ERRCODE_INVALID_DB_LIST"))
        self.assertFalse(hasattr(Security, "CSSM_ERRCODE_INVALID_DB_LIST_POINTER"))
        self.assertFalse(hasattr(Security, "CSSM_ERRCODE_UNKNOWN_FORMAT"))
        self.assertFalse(hasattr(Security, "CSSM_ERRCODE_UNKNOWN_TAG"))
        self.assertFalse(hasattr(Security, "CSSM_ERRCODE_INVALID_CSP_HANDLE"))
        self.assertFalse(hasattr(Security, "CSSM_ERRCODE_INVALID_DL_HANDLE"))
        self.assertFalse(hasattr(Security, "CSSM_ERRCODE_INVALID_CL_HANDLE"))
        self.assertFalse(hasattr(Security, "CSSM_ERRCODE_INVALID_TP_HANDLE"))
        self.assertFalse(hasattr(Security, "CSSM_ERRCODE_INVALID_KR_HANDLE"))
        self.assertFalse(hasattr(Security, "CSSM_ERRCODE_INVALID_AC_HANDLE"))
        self.assertFalse(hasattr(Security, "CSSM_ERRCODE_INVALID_PASSTHROUGH_ID"))
        self.assertFalse(hasattr(Security, "CSSM_ERRCODE_INVALID_NETWORK_ADDR"))
        self.assertFalse(hasattr(Security, "CSSM_ERRCODE_INVALID_CRYPTO_DATA"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSSM_INTERNAL_ERROR"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSSM_MEMORY_ERROR"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSSM_MDS_ERROR"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSSM_INVALID_POINTER"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSSM_INVALID_INPUT_POINTER"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSSM_INVALID_OUTPUT_POINTER"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSSM_FUNCTION_NOT_IMPLEMENTED"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSSM_SELF_CHECK_FAILED"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSSM_OS_ACCESS_DENIED"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSSM_FUNCTION_FAILED"))
        self.assertFalse(
            hasattr(Security, "CSSMERR_CSSM_MODULE_MANIFEST_VERIFY_FAILED")
        )
        self.assertFalse(hasattr(Security, "CSSMERR_CSSM_INVALID_GUID"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSSM_INVALID_CONTEXT_HANDLE"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSSM_INCOMPATIBLE_VERSION"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSSM_PRIVILEGE_NOT_GRANTED"))
        self.assertFalse(hasattr(Security, "CSSM_CSSM_BASE_CSSM_ERROR"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSSM_SCOPE_NOT_SUPPORTED"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSSM_PVC_ALREADY_CONFIGURED"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSSM_INVALID_PVC"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSSM_EMM_LOAD_FAILED"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSSM_EMM_UNLOAD_FAILED"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSSM_ADDIN_LOAD_FAILED"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSSM_INVALID_KEY_HIERARCHY"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSSM_ADDIN_UNLOAD_FAILED"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSSM_LIB_REF_NOT_FOUND"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSSM_INVALID_ADDIN_FUNCTION_TABLE"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSSM_EMM_AUTHENTICATE_FAILED"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSSM_ADDIN_AUTHENTICATE_FAILED"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSSM_INVALID_SERVICE_MASK"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSSM_MODULE_NOT_LOADED"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSSM_INVALID_SUBSERVICEID"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSSM_BUFFER_TOO_SMALL"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSSM_INVALID_ATTRIBUTE"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSSM_ATTRIBUTE_NOT_IN_CONTEXT"))
        self.assertFalse(
            hasattr(Security, "CSSMERR_CSSM_MODULE_MANAGER_INITIALIZE_FAIL")
        )
        self.assertFalse(hasattr(Security, "CSSMERR_CSSM_MODULE_MANAGER_NOT_FOUND"))
        self.assertFalse(
            hasattr(Security, "CSSMERR_CSSM_EVENT_NOTIFICATION_CALLBACK_NOT_FOUND")
        )
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_INTERNAL_ERROR"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_MEMORY_ERROR"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_MDS_ERROR"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_INVALID_POINTER"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_INVALID_INPUT_POINTER"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_INVALID_OUTPUT_POINTER"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_FUNCTION_NOT_IMPLEMENTED"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_SELF_CHECK_FAILED"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_OS_ACCESS_DENIED"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_FUNCTION_FAILED"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_OPERATION_AUTH_DENIED"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_OBJECT_USE_AUTH_DENIED"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_OBJECT_MANIP_AUTH_DENIED"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_OBJECT_ACL_NOT_SUPPORTED"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_OBJECT_ACL_REQUIRED"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_INVALID_ACCESS_CREDENTIALS"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_INVALID_ACL_BASE_CERTS"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_ACL_BASE_CERTS_NOT_SUPPORTED"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_INVALID_SAMPLE_VALUE"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_SAMPLE_VALUE_NOT_SUPPORTED"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_INVALID_ACL_SUBJECT_VALUE"))
        self.assertFalse(
            hasattr(Security, "CSSMERR_CSP_ACL_SUBJECT_TYPE_NOT_SUPPORTED")
        )
        self.assertFalse(
            hasattr(Security, "CSSMERR_CSP_INVALID_ACL_CHALLENGE_CALLBACK")
        )
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_ACL_CHALLENGE_CALLBACK_FAILED"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_INVALID_ACL_ENTRY_TAG"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_ACL_ENTRY_TAG_NOT_FOUND"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_INVALID_ACL_EDIT_MODE"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_ACL_CHANGE_FAILED"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_INVALID_NEW_ACL_ENTRY"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_INVALID_NEW_ACL_OWNER"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_ACL_DELETE_FAILED"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_ACL_REPLACE_FAILED"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_ACL_ADD_FAILED"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_INVALID_CONTEXT_HANDLE"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_PRIVILEGE_NOT_GRANTED"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_INVALID_DATA"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_INVALID_PASSTHROUGH_ID"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_INVALID_CRYPTO_DATA"))
        self.assertFalse(hasattr(Security, "CSSM_CSP_BASE_CSP_ERROR"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_INPUT_LENGTH_ERROR"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_OUTPUT_LENGTH_ERROR"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_PRIVILEGE_NOT_SUPPORTED"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_DEVICE_ERROR"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_DEVICE_MEMORY_ERROR"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_ATTACH_HANDLE_BUSY"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_NOT_LOGGED_IN"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_INVALID_KEY"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_INVALID_KEY_REFERENCE"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_INVALID_KEY_CLASS"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_ALGID_MISMATCH"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_KEY_USAGE_INCORRECT"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_KEY_BLOB_TYPE_INCORRECT"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_KEY_HEADER_INCONSISTENT"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_UNSUPPORTED_KEY_FORMAT"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_UNSUPPORTED_KEY_SIZE"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_INVALID_KEY_POINTER"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_INVALID_KEYUSAGE_MASK"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_UNSUPPORTED_KEYUSAGE_MASK"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_INVALID_KEYATTR_MASK"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_UNSUPPORTED_KEYATTR_MASK"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_INVALID_KEY_LABEL"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_UNSUPPORTED_KEY_LABEL"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_INVALID_KEY_FORMAT"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_INVALID_DATA_COUNT"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_VECTOR_OF_BUFS_UNSUPPORTED"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_INVALID_INPUT_VECTOR"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_INVALID_OUTPUT_VECTOR"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_INVALID_CONTEXT"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_INVALID_ALGORITHM"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_INVALID_ATTR_KEY"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_MISSING_ATTR_KEY"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_INVALID_ATTR_INIT_VECTOR"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_MISSING_ATTR_INIT_VECTOR"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_INVALID_ATTR_SALT"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_MISSING_ATTR_SALT"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_INVALID_ATTR_PADDING"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_MISSING_ATTR_PADDING"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_INVALID_ATTR_RANDOM"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_MISSING_ATTR_RANDOM"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_INVALID_ATTR_SEED"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_MISSING_ATTR_SEED"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_INVALID_ATTR_PASSPHRASE"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_MISSING_ATTR_PASSPHRASE"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_INVALID_ATTR_KEY_LENGTH"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_MISSING_ATTR_KEY_LENGTH"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_INVALID_ATTR_BLOCK_SIZE"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_MISSING_ATTR_BLOCK_SIZE"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_INVALID_ATTR_OUTPUT_SIZE"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_MISSING_ATTR_OUTPUT_SIZE"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_INVALID_ATTR_ROUNDS"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_MISSING_ATTR_ROUNDS"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_INVALID_ATTR_ALG_PARAMS"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_MISSING_ATTR_ALG_PARAMS"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_INVALID_ATTR_LABEL"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_MISSING_ATTR_LABEL"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_INVALID_ATTR_KEY_TYPE"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_MISSING_ATTR_KEY_TYPE"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_INVALID_ATTR_MODE"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_MISSING_ATTR_MODE"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_INVALID_ATTR_EFFECTIVE_BITS"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_MISSING_ATTR_EFFECTIVE_BITS"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_INVALID_ATTR_START_DATE"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_MISSING_ATTR_START_DATE"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_INVALID_ATTR_END_DATE"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_MISSING_ATTR_END_DATE"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_INVALID_ATTR_VERSION"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_MISSING_ATTR_VERSION"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_INVALID_ATTR_PRIME"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_MISSING_ATTR_PRIME"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_INVALID_ATTR_BASE"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_MISSING_ATTR_BASE"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_INVALID_ATTR_SUBPRIME"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_MISSING_ATTR_SUBPRIME"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_INVALID_ATTR_ITERATION_COUNT"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_MISSING_ATTR_ITERATION_COUNT"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_INVALID_ATTR_DL_DB_HANDLE"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_MISSING_ATTR_DL_DB_HANDLE"))
        self.assertFalse(
            hasattr(Security, "CSSMERR_CSP_INVALID_ATTR_ACCESS_CREDENTIALS")
        )
        self.assertFalse(
            hasattr(Security, "CSSMERR_CSP_MISSING_ATTR_ACCESS_CREDENTIALS")
        )
        self.assertFalse(
            hasattr(Security, "CSSMERR_CSP_INVALID_ATTR_PUBLIC_KEY_FORMAT")
        )
        self.assertFalse(
            hasattr(Security, "CSSMERR_CSP_MISSING_ATTR_PUBLIC_KEY_FORMAT")
        )
        self.assertFalse(
            hasattr(Security, "CSSMERR_CSP_INVALID_ATTR_PRIVATE_KEY_FORMAT")
        )
        self.assertFalse(
            hasattr(Security, "CSSMERR_CSP_MISSING_ATTR_PRIVATE_KEY_FORMAT")
        )
        self.assertFalse(
            hasattr(Security, "CSSMERR_CSP_INVALID_ATTR_SYMMETRIC_KEY_FORMAT")
        )
        self.assertFalse(
            hasattr(Security, "CSSMERR_CSP_MISSING_ATTR_SYMMETRIC_KEY_FORMAT")
        )
        self.assertFalse(
            hasattr(Security, "CSSMERR_CSP_INVALID_ATTR_WRAPPED_KEY_FORMAT")
        )
        self.assertFalse(
            hasattr(Security, "CSSMERR_CSP_MISSING_ATTR_WRAPPED_KEY_FORMAT")
        )
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_STAGED_OPERATION_IN_PROGRESS"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_STAGED_OPERATION_NOT_STARTED"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_VERIFY_FAILED"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_INVALID_SIGNATURE"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_QUERY_SIZE_UNKNOWN"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_BLOCK_SIZE_MISMATCH"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_PRIVATE_KEY_NOT_FOUND"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_PUBLIC_KEY_INCONSISTENT"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_DEVICE_VERIFY_FAILED"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_INVALID_LOGIN_NAME"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_ALREADY_LOGGED_IN"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_PRIVATE_KEY_ALREADY_EXISTS"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_KEY_LABEL_ALREADY_EXISTS"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_INVALID_DIGEST_ALGORITHM"))
        self.assertFalse(hasattr(Security, "CSSMERR_CSP_CRYPTO_DATA_CALLBACK_FAILED"))
        self.assertFalse(hasattr(Security, "CSSMERR_TP_INTERNAL_ERROR"))
        self.assertFalse(hasattr(Security, "CSSMERR_TP_MEMORY_ERROR"))
        self.assertFalse(hasattr(Security, "CSSMERR_TP_MDS_ERROR"))
        self.assertFalse(hasattr(Security, "CSSMERR_TP_INVALID_POINTER"))
        self.assertFalse(hasattr(Security, "CSSMERR_TP_INVALID_INPUT_POINTER"))
        self.assertFalse(hasattr(Security, "CSSMERR_TP_INVALID_OUTPUT_POINTER"))
        self.assertFalse(hasattr(Security, "CSSMERR_TP_FUNCTION_NOT_IMPLEMENTED"))
        self.assertFalse(hasattr(Security, "CSSMERR_TP_SELF_CHECK_FAILED"))
        self.assertFalse(hasattr(Security, "CSSMERR_TP_OS_ACCESS_DENIED"))
        self.assertFalse(hasattr(Security, "CSSMERR_TP_FUNCTION_FAILED"))
        self.assertFalse(hasattr(Security, "CSSMERR_TP_INVALID_CONTEXT_HANDLE"))
        self.assertFalse(hasattr(Security, "CSSMERR_TP_INVALID_DATA"))
        self.assertFalse(hasattr(Security, "CSSMERR_TP_INVALID_DB_LIST"))
        self.assertFalse(hasattr(Security, "CSSMERR_TP_INVALID_CERTGROUP_POINTER"))
        self.assertFalse(hasattr(Security, "CSSMERR_TP_INVALID_CERT_POINTER"))
        self.assertFalse(hasattr(Security, "CSSMERR_TP_INVALID_CRL_POINTER"))
        self.assertFalse(hasattr(Security, "CSSMERR_TP_INVALID_FIELD_POINTER"))
        self.assertFalse(hasattr(Security, "CSSMERR_TP_INVALID_NETWORK_ADDR"))
        self.assertFalse(hasattr(Security, "CSSMERR_TP_CRL_ALREADY_SIGNED"))
        self.assertFalse(hasattr(Security, "CSSMERR_TP_INVALID_NUMBER_OF_FIELDS"))
        self.assertFalse(hasattr(Security, "CSSMERR_TP_VERIFICATION_FAILURE"))
        self.assertFalse(hasattr(Security, "CSSMERR_TP_INVALID_DB_HANDLE"))
        self.assertFalse(hasattr(Security, "CSSMERR_TP_UNKNOWN_FORMAT"))
        self.assertFalse(hasattr(Security, "CSSMERR_TP_UNKNOWN_TAG"))
        self.assertFalse(hasattr(Security, "CSSMERR_TP_INVALID_PASSTHROUGH_ID"))
        self.assertFalse(hasattr(Security, "CSSMERR_TP_INVALID_CSP_HANDLE"))
        self.assertFalse(hasattr(Security, "CSSMERR_TP_INVALID_DL_HANDLE"))
        self.assertFalse(hasattr(Security, "CSSMERR_TP_INVALID_CL_HANDLE"))
        self.assertFalse(hasattr(Security, "CSSMERR_TP_INVALID_DB_LIST_POINTER"))
        self.assertFalse(hasattr(Security, "CSSM_TP_BASE_TP_ERROR"))
        self.assertFalse(
            hasattr(Security, "CSSMERR_TP_INVALID_CALLERAUTH_CONTEXT_POINTER")
        )
        self.assertFalse(hasattr(Security, "CSSMERR_TP_INVALID_IDENTIFIER_POINTER"))
        self.assertFalse(hasattr(Security, "CSSMERR_TP_INVALID_KEYCACHE_HANDLE"))
        self.assertFalse(hasattr(Security, "CSSMERR_TP_INVALID_CERTGROUP"))
        self.assertFalse(hasattr(Security, "CSSMERR_TP_INVALID_CRLGROUP"))
        self.assertFalse(hasattr(Security, "CSSMERR_TP_INVALID_CRLGROUP_POINTER"))
        self.assertFalse(hasattr(Security, "CSSMERR_TP_AUTHENTICATION_FAILED"))
        self.assertFalse(hasattr(Security, "CSSMERR_TP_CERTGROUP_INCOMPLETE"))
        self.assertFalse(hasattr(Security, "CSSMERR_TP_CERTIFICATE_CANT_OPERATE"))
        self.assertFalse(hasattr(Security, "CSSMERR_TP_CERT_EXPIRED"))
        self.assertFalse(hasattr(Security, "CSSMERR_TP_CERT_NOT_VALID_YET"))
        self.assertFalse(hasattr(Security, "CSSMERR_TP_CERT_REVOKED"))
        self.assertFalse(hasattr(Security, "CSSMERR_TP_CERT_SUSPENDED"))
        self.assertFalse(hasattr(Security, "CSSMERR_TP_INSUFFICIENT_CREDENTIALS"))
        self.assertFalse(hasattr(Security, "CSSMERR_TP_INVALID_ACTION"))
        self.assertFalse(hasattr(Security, "CSSMERR_TP_INVALID_ACTION_DATA"))
        self.assertFalse(hasattr(Security, "CSSMERR_TP_INVALID_ANCHOR_CERT"))
        self.assertFalse(hasattr(Security, "CSSMERR_TP_INVALID_AUTHORITY"))
        self.assertFalse(hasattr(Security, "CSSMERR_TP_VERIFY_ACTION_FAILED"))
        self.assertFalse(hasattr(Security, "CSSMERR_TP_INVALID_CERTIFICATE"))
        self.assertFalse(hasattr(Security, "CSSMERR_TP_INVALID_CERT_AUTHORITY"))
        self.assertFalse(hasattr(Security, "CSSMERR_TP_INVALID_CRL_AUTHORITY"))
        self.assertFalse(hasattr(Security, "CSSMERR_TP_INVALID_CRL_ENCODING"))
        self.assertFalse(hasattr(Security, "CSSMERR_TP_INVALID_CRL_TYPE"))
        self.assertFalse(hasattr(Security, "CSSMERR_TP_INVALID_CRL"))
        self.assertFalse(hasattr(Security, "CSSMERR_TP_INVALID_FORM_TYPE"))
        self.assertFalse(hasattr(Security, "CSSMERR_TP_INVALID_ID"))
        self.assertFalse(hasattr(Security, "CSSMERR_TP_INVALID_IDENTIFIER"))
        self.assertFalse(hasattr(Security, "CSSMERR_TP_INVALID_INDEX"))
        self.assertFalse(hasattr(Security, "CSSMERR_TP_INVALID_NAME"))
        self.assertFalse(hasattr(Security, "CSSMERR_TP_INVALID_POLICY_IDENTIFIERS"))
        self.assertFalse(hasattr(Security, "CSSMERR_TP_INVALID_TIMESTRING"))
        self.assertFalse(hasattr(Security, "CSSMERR_TP_INVALID_REASON"))
        self.assertFalse(hasattr(Security, "CSSMERR_TP_INVALID_REQUEST_INPUTS"))
        self.assertFalse(hasattr(Security, "CSSMERR_TP_INVALID_RESPONSE_VECTOR"))
        self.assertFalse(hasattr(Security, "CSSMERR_TP_INVALID_SIGNATURE"))
        self.assertFalse(hasattr(Security, "CSSMERR_TP_INVALID_STOP_ON_POLICY"))
        self.assertFalse(hasattr(Security, "CSSMERR_TP_INVALID_CALLBACK"))
        self.assertFalse(hasattr(Security, "CSSMERR_TP_INVALID_TUPLE"))
        self.assertFalse(hasattr(Security, "CSSMERR_TP_NOT_SIGNER"))
        self.assertFalse(hasattr(Security, "CSSMERR_TP_NOT_TRUSTED"))
        self.assertFalse(hasattr(Security, "CSSMERR_TP_NO_DEFAULT_AUTHORITY"))
        self.assertFalse(hasattr(Security, "CSSMERR_TP_REJECTED_FORM"))
        self.assertFalse(hasattr(Security, "CSSMERR_TP_REQUEST_LOST"))
        self.assertFalse(hasattr(Security, "CSSMERR_TP_REQUEST_REJECTED"))
        self.assertFalse(hasattr(Security, "CSSMERR_TP_UNSUPPORTED_ADDR_TYPE"))
        self.assertFalse(hasattr(Security, "CSSMERR_TP_UNSUPPORTED_SERVICE"))
        self.assertFalse(hasattr(Security, "CSSMERR_TP_INVALID_TUPLEGROUP_POINTER"))
        self.assertFalse(hasattr(Security, "CSSMERR_TP_INVALID_TUPLEGROUP"))
        self.assertFalse(hasattr(Security, "CSSMERR_AC_INTERNAL_ERROR"))
        self.assertFalse(hasattr(Security, "CSSMERR_AC_MEMORY_ERROR"))
        self.assertFalse(hasattr(Security, "CSSMERR_AC_MDS_ERROR"))
        self.assertFalse(hasattr(Security, "CSSMERR_AC_INVALID_POINTER"))
        self.assertFalse(hasattr(Security, "CSSMERR_AC_INVALID_INPUT_POINTER"))
        self.assertFalse(hasattr(Security, "CSSMERR_AC_INVALID_OUTPUT_POINTER"))
        self.assertFalse(hasattr(Security, "CSSMERR_AC_FUNCTION_NOT_IMPLEMENTED"))
        self.assertFalse(hasattr(Security, "CSSMERR_AC_SELF_CHECK_FAILED"))
        self.assertFalse(hasattr(Security, "CSSMERR_AC_OS_ACCESS_DENIED"))
        self.assertFalse(hasattr(Security, "CSSMERR_AC_FUNCTION_FAILED"))
        self.assertFalse(hasattr(Security, "CSSMERR_AC_INVALID_CONTEXT_HANDLE"))
        self.assertFalse(hasattr(Security, "CSSMERR_AC_INVALID_DATA"))
        self.assertFalse(hasattr(Security, "CSSMERR_AC_INVALID_DB_LIST"))
        self.assertFalse(hasattr(Security, "CSSMERR_AC_INVALID_PASSTHROUGH_ID"))
        self.assertFalse(hasattr(Security, "CSSMERR_AC_INVALID_DL_HANDLE"))
        self.assertFalse(hasattr(Security, "CSSMERR_AC_INVALID_CL_HANDLE"))
        self.assertFalse(hasattr(Security, "CSSMERR_AC_INVALID_TP_HANDLE"))
        self.assertFalse(hasattr(Security, "CSSMERR_AC_INVALID_DB_HANDLE"))
        self.assertFalse(hasattr(Security, "CSSMERR_AC_INVALID_DB_LIST_POINTER"))
        self.assertFalse(hasattr(Security, "CSSM_AC_BASE_AC_ERROR"))
        self.assertFalse(hasattr(Security, "CSSMERR_AC_INVALID_BASE_ACLS"))
        self.assertFalse(hasattr(Security, "CSSMERR_AC_INVALID_TUPLE_CREDENTIALS"))
        self.assertFalse(hasattr(Security, "CSSMERR_AC_INVALID_ENCODING"))
        self.assertFalse(hasattr(Security, "CSSMERR_AC_INVALID_VALIDITY_PERIOD"))
        self.assertFalse(hasattr(Security, "CSSMERR_AC_INVALID_REQUESTOR"))
        self.assertFalse(hasattr(Security, "CSSMERR_AC_INVALID_REQUEST_DESCRIPTOR"))
        self.assertFalse(hasattr(Security, "CSSMERR_CL_INTERNAL_ERROR"))
        self.assertFalse(hasattr(Security, "CSSMERR_CL_MEMORY_ERROR"))
        self.assertFalse(hasattr(Security, "CSSMERR_CL_MDS_ERROR"))
        self.assertFalse(hasattr(Security, "CSSMERR_CL_INVALID_POINTER"))
        self.assertFalse(hasattr(Security, "CSSMERR_CL_INVALID_INPUT_POINTER"))
        self.assertFalse(hasattr(Security, "CSSMERR_CL_INVALID_OUTPUT_POINTER"))
        self.assertFalse(hasattr(Security, "CSSMERR_CL_FUNCTION_NOT_IMPLEMENTED"))
        self.assertFalse(hasattr(Security, "CSSMERR_CL_SELF_CHECK_FAILED"))
        self.assertFalse(hasattr(Security, "CSSMERR_CL_OS_ACCESS_DENIED"))
        self.assertFalse(hasattr(Security, "CSSMERR_CL_FUNCTION_FAILED"))
        self.assertFalse(hasattr(Security, "CSSMERR_CL_INVALID_CONTEXT_HANDLE"))
        self.assertFalse(hasattr(Security, "CSSMERR_CL_INVALID_CERTGROUP_POINTER"))
        self.assertFalse(hasattr(Security, "CSSMERR_CL_INVALID_CERT_POINTER"))
        self.assertFalse(hasattr(Security, "CSSMERR_CL_INVALID_CRL_POINTER"))
        self.assertFalse(hasattr(Security, "CSSMERR_CL_INVALID_FIELD_POINTER"))
        self.assertFalse(hasattr(Security, "CSSMERR_CL_INVALID_DATA"))
        self.assertFalse(hasattr(Security, "CSSMERR_CL_CRL_ALREADY_SIGNED"))
        self.assertFalse(hasattr(Security, "CSSMERR_CL_INVALID_NUMBER_OF_FIELDS"))
        self.assertFalse(hasattr(Security, "CSSMERR_CL_VERIFICATION_FAILURE"))
        self.assertFalse(hasattr(Security, "CSSMERR_CL_UNKNOWN_FORMAT"))
        self.assertFalse(hasattr(Security, "CSSMERR_CL_UNKNOWN_TAG"))
        self.assertFalse(hasattr(Security, "CSSMERR_CL_INVALID_PASSTHROUGH_ID"))
        self.assertFalse(hasattr(Security, "CSSM_CL_BASE_CL_ERROR"))
        self.assertFalse(hasattr(Security, "CSSMERR_CL_INVALID_BUNDLE_POINTER"))
        self.assertFalse(hasattr(Security, "CSSMERR_CL_INVALID_CACHE_HANDLE"))
        self.assertFalse(hasattr(Security, "CSSMERR_CL_INVALID_RESULTS_HANDLE"))
        self.assertFalse(hasattr(Security, "CSSMERR_CL_INVALID_BUNDLE_INFO"))
        self.assertFalse(hasattr(Security, "CSSMERR_CL_INVALID_CRL_INDEX"))
        self.assertFalse(hasattr(Security, "CSSMERR_CL_INVALID_SCOPE"))
        self.assertFalse(hasattr(Security, "CSSMERR_CL_NO_FIELD_VALUES"))
        self.assertFalse(hasattr(Security, "CSSMERR_CL_SCOPE_NOT_SUPPORTED"))
        self.assertFalse(hasattr(Security, "CSSMERR_DL_INTERNAL_ERROR"))
        self.assertFalse(hasattr(Security, "CSSMERR_DL_MEMORY_ERROR"))
        self.assertFalse(hasattr(Security, "CSSMERR_DL_MDS_ERROR"))
        self.assertFalse(hasattr(Security, "CSSMERR_DL_INVALID_POINTER"))
        self.assertFalse(hasattr(Security, "CSSMERR_DL_INVALID_INPUT_POINTER"))
        self.assertFalse(hasattr(Security, "CSSMERR_DL_INVALID_OUTPUT_POINTER"))
        self.assertFalse(hasattr(Security, "CSSMERR_DL_FUNCTION_NOT_IMPLEMENTED"))
        self.assertFalse(hasattr(Security, "CSSMERR_DL_SELF_CHECK_FAILED"))
        self.assertFalse(hasattr(Security, "CSSMERR_DL_OS_ACCESS_DENIED"))
        self.assertFalse(hasattr(Security, "CSSMERR_DL_FUNCTION_FAILED"))
        self.assertFalse(hasattr(Security, "CSSMERR_DL_INVALID_CSP_HANDLE"))
        self.assertFalse(hasattr(Security, "CSSMERR_DL_INVALID_DL_HANDLE"))
        self.assertFalse(hasattr(Security, "CSSMERR_DL_INVALID_CL_HANDLE"))
        self.assertFalse(hasattr(Security, "CSSMERR_DL_INVALID_DB_LIST_POINTER"))
        self.assertFalse(hasattr(Security, "CSSMERR_DL_OPERATION_AUTH_DENIED"))
        self.assertFalse(hasattr(Security, "CSSMERR_DL_OBJECT_USE_AUTH_DENIED"))
        self.assertFalse(hasattr(Security, "CSSMERR_DL_OBJECT_MANIP_AUTH_DENIED"))
        self.assertFalse(hasattr(Security, "CSSMERR_DL_OBJECT_ACL_NOT_SUPPORTED"))
        self.assertFalse(hasattr(Security, "CSSMERR_DL_OBJECT_ACL_REQUIRED"))
        self.assertFalse(hasattr(Security, "CSSMERR_DL_INVALID_ACCESS_CREDENTIALS"))
        self.assertFalse(hasattr(Security, "CSSMERR_DL_INVALID_ACL_BASE_CERTS"))
        self.assertFalse(hasattr(Security, "CSSMERR_DL_ACL_BASE_CERTS_NOT_SUPPORTED"))
        self.assertFalse(hasattr(Security, "CSSMERR_DL_INVALID_SAMPLE_VALUE"))
        self.assertFalse(hasattr(Security, "CSSMERR_DL_SAMPLE_VALUE_NOT_SUPPORTED"))
        self.assertFalse(hasattr(Security, "CSSMERR_DL_INVALID_ACL_SUBJECT_VALUE"))
        self.assertFalse(hasattr(Security, "CSSMERR_DL_ACL_SUBJECT_TYPE_NOT_SUPPORTED"))
        self.assertFalse(hasattr(Security, "CSSMERR_DL_INVALID_ACL_CHALLENGE_CALLBACK"))
        self.assertFalse(hasattr(Security, "CSSMERR_DL_ACL_CHALLENGE_CALLBACK_FAILED"))
        self.assertFalse(hasattr(Security, "CSSMERR_DL_INVALID_ACL_ENTRY_TAG"))
        self.assertFalse(hasattr(Security, "CSSMERR_DL_ACL_ENTRY_TAG_NOT_FOUND"))
        self.assertFalse(hasattr(Security, "CSSMERR_DL_INVALID_ACL_EDIT_MODE"))
        self.assertFalse(hasattr(Security, "CSSMERR_DL_ACL_CHANGE_FAILED"))
        self.assertFalse(hasattr(Security, "CSSMERR_DL_INVALID_NEW_ACL_ENTRY"))
        self.assertFalse(hasattr(Security, "CSSMERR_DL_INVALID_NEW_ACL_OWNER"))
        self.assertFalse(hasattr(Security, "CSSMERR_DL_ACL_DELETE_FAILED"))
        self.assertFalse(hasattr(Security, "CSSMERR_DL_ACL_REPLACE_FAILED"))
        self.assertFalse(hasattr(Security, "CSSMERR_DL_ACL_ADD_FAILED"))
        self.assertFalse(hasattr(Security, "CSSMERR_DL_INVALID_DB_HANDLE"))
        self.assertFalse(hasattr(Security, "CSSMERR_DL_INVALID_PASSTHROUGH_ID"))
        self.assertFalse(hasattr(Security, "CSSMERR_DL_INVALID_NETWORK_ADDR"))
        self.assertFalse(hasattr(Security, "CSSM_DL_BASE_DL_ERROR"))
        self.assertFalse(hasattr(Security, "CSSMERR_DL_DATABASE_CORRUPT"))
        self.assertFalse(hasattr(Security, "CSSMERR_DL_INVALID_RECORD_INDEX"))
        self.assertFalse(hasattr(Security, "CSSMERR_DL_INVALID_RECORDTYPE"))
        self.assertFalse(hasattr(Security, "CSSMERR_DL_INVALID_FIELD_NAME"))
        self.assertFalse(hasattr(Security, "CSSMERR_DL_UNSUPPORTED_FIELD_FORMAT"))
        self.assertFalse(hasattr(Security, "CSSMERR_DL_UNSUPPORTED_INDEX_INFO"))
        self.assertFalse(hasattr(Security, "CSSMERR_DL_UNSUPPORTED_LOCALITY"))
        self.assertFalse(hasattr(Security, "CSSMERR_DL_UNSUPPORTED_NUM_ATTRIBUTES"))
        self.assertFalse(hasattr(Security, "CSSMERR_DL_UNSUPPORTED_NUM_INDEXES"))
        self.assertFalse(hasattr(Security, "CSSMERR_DL_UNSUPPORTED_NUM_RECORDTYPES"))
        self.assertFalse(hasattr(Security, "CSSMERR_DL_UNSUPPORTED_RECORDTYPE"))
        self.assertFalse(hasattr(Security, "CSSMERR_DL_FIELD_SPECIFIED_MULTIPLE"))
        self.assertFalse(hasattr(Security, "CSSMERR_DL_INCOMPATIBLE_FIELD_FORMAT"))
        self.assertFalse(hasattr(Security, "CSSMERR_DL_INVALID_PARSING_MODULE"))
        self.assertFalse(hasattr(Security, "CSSMERR_DL_INVALID_DB_NAME"))
        self.assertFalse(hasattr(Security, "CSSMERR_DL_DATASTORE_DOESNOT_EXIST"))
        self.assertFalse(hasattr(Security, "CSSMERR_DL_DATASTORE_ALREADY_EXISTS"))
        self.assertFalse(hasattr(Security, "CSSMERR_DL_DB_LOCKED"))
        self.assertFalse(hasattr(Security, "CSSMERR_DL_DATASTORE_IS_OPEN"))
        self.assertFalse(hasattr(Security, "CSSMERR_DL_RECORD_NOT_FOUND"))
        self.assertFalse(hasattr(Security, "CSSMERR_DL_MISSING_VALUE"))
        self.assertFalse(hasattr(Security, "CSSMERR_DL_UNSUPPORTED_QUERY"))
        self.assertFalse(hasattr(Security, "CSSMERR_DL_UNSUPPORTED_QUERY_LIMITS"))
        self.assertFalse(
            hasattr(Security, "CSSMERR_DL_UNSUPPORTED_NUM_SELECTION_PREDS")
        )
        self.assertFalse(hasattr(Security, "CSSMERR_DL_UNSUPPORTED_OPERATOR"))
        self.assertFalse(hasattr(Security, "CSSMERR_DL_INVALID_RESULTS_HANDLE"))
        self.assertFalse(hasattr(Security, "CSSMERR_DL_INVALID_DB_LOCATION"))
        self.assertFalse(hasattr(Security, "CSSMERR_DL_INVALID_ACCESS_REQUEST"))
        self.assertFalse(hasattr(Security, "CSSMERR_DL_INVALID_INDEX_INFO"))
        self.assertFalse(hasattr(Security, "CSSMERR_DL_INVALID_SELECTION_TAG"))
        self.assertFalse(hasattr(Security, "CSSMERR_DL_INVALID_NEW_OWNER"))
        self.assertFalse(hasattr(Security, "CSSMERR_DL_INVALID_RECORD_UID"))
        self.assertFalse(hasattr(Security, "CSSMERR_DL_INVALID_UNIQUE_INDEX_DATA"))
        self.assertFalse(hasattr(Security, "CSSMERR_DL_INVALID_MODIFY_MODE"))
        self.assertFalse(hasattr(Security, "CSSMERR_DL_INVALID_OPEN_PARAMETERS"))
        self.assertFalse(hasattr(Security, "CSSMERR_DL_RECORD_MODIFIED"))
        self.assertFalse(hasattr(Security, "CSSMERR_DL_ENDOFDATA"))
        self.assertFalse(hasattr(Security, "CSSMERR_DL_INVALID_QUERY"))
        self.assertFalse(hasattr(Security, "CSSMERR_DL_INVALID_VALUE"))
        self.assertFalse(hasattr(Security, "CSSMERR_DL_MULTIPLE_VALUES_UNSUPPORTED"))
        self.assertFalse(hasattr(Security, "CSSMERR_DL_STALE_UNIQUE_RECORD"))



import Security
from PyObjCTools.TestSupport import TestCase


class TestSecIdentitySearch(TestCase):
    def test_unsuppported(self):
        self.assertFalse(hasattr(Security, "SecIdentitySearchRef"))
        self.assertFalse(hasattr(Security, "SecIdentitySearchCreate"))
        self.assertFalse(hasattr(Security, "SecIdentitySearchCopyNext"))



import Security
from PyObjCTools.TestSupport import TestCase, min_os_level
import objc


class TestSecTransform(TestCase):
    def test_constants(self):
        self.assertIsInstance(Security.kSecTransformErrorDomain, str)
        self.assertIsInstance(Security.kSecTransformPreviousErrorKey, str)
        self.assertIsInstance(Security.kSecTransformAbortOriginatorKey, str)

        self.assertEqual(Security.kSecTransformErrorAttributeNotFound, 1)
        self.assertEqual(Security.kSecTransformErrorInvalidOperation, 2)
        self.assertEqual(Security.kSecTransformErrorNotInitializedCorrectly, 3)
        self.assertEqual(Security.kSecTransformErrorMoreThanOneOutput, 4)
        self.assertEqual(Security.kSecTransformErrorInvalidInputDictionary, 5)
        self.assertEqual(Security.kSecTransformErrorInvalidAlgorithm, 6)
        self.assertEqual(Security.kSecTransformErrorInvalidLength, 7)
        self.assertEqual(Security.kSecTransformErrorInvalidType, 8)
        self.assertEqual(Security.kSecTransformErrorInvalidInput, 10)
        self.assertEqual(Security.kSecTransformErrorNameAlreadyRegistered, 11)
        self.assertEqual(Security.kSecTransformErrorUnsupportedAttribute, 12)
        self.assertEqual(Security.kSecTransformOperationNotSupportedOnGroup, 13)
        self.assertEqual(Security.kSecTransformErrorMissingParameter, 14)
        self.assertEqual(Security.kSecTransformErrorInvalidConnection, 15)
        self.assertEqual(Security.kSecTransformTransformIsExecuting, 16)
        self.assertEqual(Security.kSecTransformInvalidOverride, 17)
        self.assertEqual(Security.kSecTransformTransformIsNotRegistered, 18)
        self.assertEqual(Security.kSecTransformErrorAbortInProgress, 19)
        self.assertEqual(Security.kSecTransformErrorAborted, 20)
        self.assertEqual(Security.kSecTransformInvalidArgument, 21)

    @min_os_level("10.7")
    def test_constants10_7(self):
        self.assertIsInstance(Security.kSecTransformInputAttributeName, str)
        self.assertIsInstance(Security.kSecTransformOutputAttributeName, str)
        self.assertIsInstance(Security.kSecTransformDebugAttributeName, str)
        self.assertIsInstance(Security.kSecTransformTransformName, str)
        self.assertIsInstance(Security.kSecTransformAbortAttributeName, str)

    @min_os_level("10.7")
    def test_functions10_7(self):
        self.assertResultHasType(
            Security.SecTransformCreateFromExternalRepresentation, objc._C_ID
        )
        self.assertResultIsCFRetained(
            Security.SecTransformCreateFromExternalRepresentation
        )
        self.assertArgHasType(
            Security.SecTransformCreateFromExternalRepresentation, 0, objc._C_ID
        )
        self.assertArgHasType(
            Security.SecTransformCreateFromExternalRepresentation,
            1,
            objc._C_OUT + objc._C_PTR + objc._C_ID,
        )

        self.assertResultHasType(
            Security.SecTransformCopyExternalRepresentation, objc._C_ID
        )
        self.assertResultIsCFRetained(Security.SecTransformCopyExternalRepresentation)
        self.assertArgHasType(
            Security.SecTransformCopyExternalRepresentation, 0, objc._C_ID
        )

        self.assertResultHasType(Security.SecTransformCreateGroupTransform, objc._C_ID)
        self.assertResultIsCFRetained(Security.SecTransformCreateGroupTransform)

        self.assertResultHasType(Security.SecTransformConnectTransforms, objc._C_ID)
        self.assertArgHasType(Security.SecTransformConnectTransforms, 0, objc._C_ID)
        self.assertArgHasType(Security.SecTransformConnectTransforms, 1, objc._C_ID)
        self.assertArgHasType(Security.SecTransformConnectTransforms, 2, objc._C_ID)
        self.assertArgHasType(Security.SecTransformConnectTransforms, 3, objc._C_ID)
        self.assertArgHasType(Security.SecTransformConnectTransforms, 4, objc._C_ID)
        self.assertArgHasType(
            Security.SecTransformConnectTransforms,
            5,
            objc._C_OUT + objc._C_PTR + objc._C_ID,
        )

        self.assertResultHasType(Security.SecTransformSetAttribute, objc._C_NSBOOL)
        self.assertArgHasType(Security.SecTransformSetAttribute, 0, objc._C_ID)
        self.assertArgHasType(Security.SecTransformSetAttribute, 1, objc._C_ID)
        self.assertArgHasType(Security.SecTransformSetAttribute, 2, objc._C_ID)
        self.assertArgHasType(
            Security.SecTransformSetAttribute, 3, objc._C_OUT + objc._C_PTR + objc._C_ID
        )

        self.assertResultHasType(Security.SecTransformGetAttribute, objc._C_ID)
        self.assertArgHasType(Security.SecTransformGetAttribute, 0, objc._C_ID)
        self.assertArgHasType(Security.SecTransformGetAttribute, 1, objc._C_ID)

        self.assertResultHasType(Security.SecTransformFindByName, objc._C_ID)
        self.assertArgHasType(Security.SecTransformFindByName, 0, objc._C_ID)
        self.assertArgHasType(Security.SecTransformFindByName, 1, objc._C_ID)

        self.assertResultHasType(Security.SecTransformExecute, objc._C_ID)
        self.assertResultIsCFRetained(Security.SecTransformExecute)
        self.assertArgHasType(Security.SecTransformExecute, 0, objc._C_ID)
        self.assertArgHasType(
            Security.SecTransformExecute, 1, objc._C_OUT + objc._C_PTR + objc._C_ID
        )

        SecMessageBlock = b"v@@Z"
        self.assertResultHasType(Security.SecTransformExecuteAsync, objc._C_VOID)
        self.assertArgHasType(Security.SecTransformExecuteAsync, 0, objc._C_ID)
        self.assertArgHasType(
            Security.SecTransformExecuteAsync, 1, b"^{dispatch_queue_s=}"
        )
        self.assertArgIsBlock(Security.SecTransformExecuteAsync, 2, SecMessageBlock)



import Security
from PyObjCTools.TestSupport import TestCase
import objc

SecureDownloadTrustSetupCallback = b"i@^v"
SecureDownloadTrustEvaluateCallback = b"i@i^v"


class TestSecureDownload(TestCase):
    def test_constants(self):
        self.assertEqual(Security.errSecureDownloadInvalidTicket, -20052)
        self.assertEqual(Security.errSecureDownloadInvalidDownload, -20053)

        self.assertEqual(Security.kSecureDownloadDoNotEvaluateSigner, 0)
        self.assertEqual(Security.kSecureDownloadEvaluateSigner, 1)
        self.assertEqual(Security.kSecureDownloadFailEvaluation, 2)

    def test_types(self):
        self.assertIsOpaquePointer(Security.SecureDownloadRef)

    def test_functions(self):
        self.assertResultHasType(Security.SecureDownloadCreateWithTicket, objc._C_INT)
        self.assertArgHasType(Security.SecureDownloadCreateWithTicket, 0, objc._C_ID)
        self.assertArgIsFunction(
            Security.SecureDownloadCreateWithTicket,
            1,
            SecureDownloadTrustSetupCallback,
            1,
        )
        self.assertArgHasType(
            Security.SecureDownloadCreateWithTicket, 2, objc._C_PTR + objc._C_VOID
        )
        self.assertArgIsFunction(
            Security.SecureDownloadCreateWithTicket,
            3,
            SecureDownloadTrustEvaluateCallback,
            1,
        )
        self.assertArgHasType(
            Security.SecureDownloadCreateWithTicket, 4, objc._C_PTR + objc._C_VOID
        )
        self.assertArgHasType(
            Security.SecureDownloadCreateWithTicket,
            5,
            objc._C_OUT + objc._C_PTR + Security.SecureDownloadRef.__typestr__,
        )

        self.assertResultHasType(Security.SecureDownloadCopyURLs, objc._C_INT)
        self.assertArgHasType(
            Security.SecureDownloadCopyURLs, 0, Security.SecureDownloadRef.__typestr__
        )
        self.assertArgHasType(
            Security.SecureDownloadCopyURLs, 1, objc._C_OUT + objc._C_PTR + objc._C_ID
        )
        self.assertArgIsCFRetained(Security.SecureDownloadCopyURLs, 1)

        self.assertResultHasType(Security.SecureDownloadCopyName, objc._C_INT)
        self.assertArgHasType(
            Security.SecureDownloadCopyName, 0, Security.SecureDownloadRef.__typestr__
        )
        self.assertArgHasType(
            Security.SecureDownloadCopyName, 1, objc._C_OUT + objc._C_PTR + objc._C_ID
        )
        self.assertArgIsCFRetained(Security.SecureDownloadCopyName, 1)

        self.assertResultHasType(Security.SecureDownloadCopyCreationDate, objc._C_INT)
        self.assertArgHasType(
            Security.SecureDownloadCopyCreationDate,
            0,
            Security.SecureDownloadRef.__typestr__,
        )
        self.assertArgHasType(
            Security.SecureDownloadCopyCreationDate,
            1,
            objc._C_OUT + objc._C_PTR + objc._C_ID,
        )
        self.assertArgIsCFRetained(Security.SecureDownloadCopyCreationDate, 1)

        self.assertResultHasType(Security.SecureDownloadGetDownloadSize, objc._C_INT)
        self.assertArgHasType(
            Security.SecureDownloadGetDownloadSize,
            0,
            Security.SecureDownloadRef.__typestr__,
        )
        self.assertArgHasType(
            Security.SecureDownloadGetDownloadSize,
            1,
            objc._C_OUT + objc._C_PTR + objc._C_LNGLNG,
        )

        self.assertResultHasType(Security.SecureDownloadUpdateWithData, objc._C_INT)
        self.assertArgHasType(
            Security.SecureDownloadUpdateWithData,
            0,
            Security.SecureDownloadRef.__typestr__,
        )
        self.assertArgHasType(Security.SecureDownloadUpdateWithData, 1, objc._C_ID)

        self.assertResultHasType(Security.SecureDownloadFinished, objc._C_INT)
        self.assertArgHasType(
            Security.SecureDownloadFinished, 0, Security.SecureDownloadRef.__typestr__
        )

        self.assertResultHasType(Security.SecureDownloadRelease, objc._C_INT)
        self.assertArgHasType(
            Security.SecureDownloadRelease, 0, Security.SecureDownloadRef.__typestr__
        )

        self.assertResultHasType(Security.SecureDownloadCopyTicketLocation, objc._C_INT)
        self.assertArgHasType(
            Security.SecureDownloadCopyTicketLocation,
            0,
            Security.SecureDownloadRef.__typestr__,
        )
        self.assertArgHasType(
            Security.SecureDownloadCopyTicketLocation,
            1,
            objc._C_OUT + objc._C_PTR + objc._C_ID,
        )
        self.assertArgIsCFRetained(Security.SecureDownloadCopyTicketLocation, 1)



import Security
from PyObjCTools.TestSupport import TestCase
import objc


class TestAuthorizationDB(TestCase):
    def test_constants(self):
        self.assertEqual(Security.kAuthorizationRightRule, b"rule")
        self.assertEqual(Security.kAuthorizationRuleIsAdmin, b"is-admin")
        self.assertEqual(
            Security.kAuthorizationRuleAuthenticateAsSessionUser,
            b"authenticate-session-owner",
        )
        self.assertEqual(
            Security.kAuthorizationRuleAuthenticateAsAdmin, b"authenticate-admin"
        )
        self.assertEqual(Security.kAuthorizationRuleClassAllow, b"allow")
        self.assertEqual(Security.kAuthorizationRuleClassDeny, b"deny")
        self.assertEqual(Security.kAuthorizationComment, b"comment")

    def test_functions(self):
        self.assertResultHasType(Security.AuthorizationRightGet, objc._C_INT)
        self.assertArgHasType(
            Security.AuthorizationRightGet,
            0,
            objc._C_IN + objc._C_PTR + objc._C_CHAR_AS_TEXT,
        )
        self.assertArgIsNullTerminated(Security.AuthorizationRightGet, 0)
        self.assertArgHasType(
            Security.AuthorizationRightGet, 1, objc._C_OUT + objc._C_PTR + objc._C_ID
        )
        self.assertArgIsRetained(
            Security.AuthorizationRightGet, 1, objc._C_OUT + objc._C_PTR + objc._C_ID
        )

        self.assertResultHasType(Security.AuthorizationRightSet, objc._C_INT)
        self.assertArgHasType(
            Security.AuthorizationRightSet, 0, Security.AuthorizationRef.__typestr__
        )
        self.assertArgHasType(
            Security.AuthorizationRightSet,
            1,
            objc._C_IN + objc._C_PTR + objc._C_CHAR_AS_TEXT,
        )
        self.assertArgIsNullTerminated(Security.AuthorizationRightSet, 1)
        self.assertArgHasType(Security.AuthorizationRightSet, 2, objc._C_ID)
        self.assertArgHasType(Security.AuthorizationRightSet, 3, objc._C_ID)
        self.assertArgHasType(Security.AuthorizationRightSet, 4, objc._C_ID)
        self.assertArgHasType(Security.AuthorizationRightSet, 5, objc._C_ID)

        self.assertResultHasType(Security.AuthorizationRightRemove, objc._C_INT)
        self.assertArgHasType(
            Security.AuthorizationRightRemove, 0, Security.AuthorizationRef.__typestr__
        )
        self.assertArgHasType(
            Security.AuthorizationRightRemove,
            1,
            objc._C_IN + objc._C_PTR + objc._C_CHAR_AS_TEXT,
        )
        self.assertArgIsNullTerminated(Security.AuthorizationRightRemove, 1)



import Security
from PyObjCTools.TestSupport import TestCase


class TestMDS(TestCase):
    def test_unsuppported(self):
        self.assertFalse(hasattr(Security, "MDS_OBJECT_DIRECTORY_NAME"))
        self.assertFalse(hasattr(Security, "MDS_CDSA_DIRECTORY_NAME"))
        self.assertFalse(hasattr(Security, "CSSM_DB_RELATIONID_MDS_START"))
        self.assertFalse(hasattr(Security, "CSSM_DB_RELATIONID_MDS_END"))
        self.assertFalse(hasattr(Security, "MDS_OBJECT_RECORDTYPE"))
        self.assertFalse(hasattr(Security, "MDS_CDSA_SCHEMA_START"))
        self.assertFalse(hasattr(Security, "MDS_CDSADIR_CSSM_RECORDTYPE"))
        self.assertFalse(hasattr(Security, "MDS_CDSADIR_KRMM_RECORDTYPE"))
        self.assertFalse(hasattr(Security, "MDS_CDSADIR_EMM_RECORDTYPE"))
        self.assertFalse(hasattr(Security, "MDS_CDSADIR_COMMON_RECORDTYPE"))
        self.assertFalse(hasattr(Security, "MDS_CDSADIR_CSP_PRIMARY_RECORDTYPE"))
        self.assertFalse(hasattr(Security, "MDS_CDSADIR_CSP_CAPABILITY_RECORDTYPE"))
        self.assertFalse(
            hasattr(Security, "MDS_CDSADIR_CSP_ENCAPSULATED_PRODUCT_RECORDTYPE")
        )
        self.assertFalse(hasattr(Security, "MDS_CDSADIR_CSP_SC_INFO_RECORDTYPE"))
        self.assertFalse(hasattr(Security, "MDS_CDSADIR_DL_PRIMARY_RECORDTYPE"))
        self.assertFalse(
            hasattr(Security, "MDS_CDSADIR_DL_ENCAPSULATED_PRODUCT_RECORDTYPE")
        )
        self.assertFalse(hasattr(Security, "MDS_CDSADIR_CL_PRIMARY_RECORDTYPE"))
        self.assertFalse(
            hasattr(Security, "MDS_CDSADIR_CL_ENCAPSULATED_PRODUCT_RECORDTYPE")
        )
        self.assertFalse(hasattr(Security, "MDS_CDSADIR_TP_PRIMARY_RECORDTYPE"))
        self.assertFalse(hasattr(Security, "MDS_CDSADIR_TP_OIDS_RECORDTYPE"))
        self.assertFalse(
            hasattr(Security, "MDS_CDSADIR_TP_ENCAPSULATED_PRODUCT_RECORDTYPE")
        )
        self.assertFalse(hasattr(Security, "MDS_CDSADIR_EMM_PRIMARY_RECORDTYPE"))
        self.assertFalse(hasattr(Security, "MDS_CDSADIR_AC_PRIMARY_RECORDTYPE"))
        self.assertFalse(hasattr(Security, "MDS_CDSADIR_KR_PRIMARY_RECORDTYPE"))
        self.assertFalse(hasattr(Security, "MDS_CDSADIR_MDS_SCHEMA_RELATIONS"))
        self.assertFalse(hasattr(Security, "MDS_CDSADIR_MDS_SCHEMA_ATTRIBUTES"))
        self.assertFalse(hasattr(Security, "MDS_CDSADIR_MDS_SCHEMA_INDEXES"))
        self.assertFalse(hasattr(Security, "CSSM_DB_ATTRIBUTE_MDS_START"))
        self.assertFalse(hasattr(Security, "CSSM_DB_ATTRIBUTE_MDS_END"))
        self.assertFalse(hasattr(Security, "MDS_CDSAATTR_MODULE_ID"))
        self.assertFalse(hasattr(Security, "MDS_CDSAATTR_MANIFEST"))
        self.assertFalse(hasattr(Security, "MDS_CDSAATTR_MODULE_NAME"))
        self.assertFalse(hasattr(Security, "MDS_CDSAATTR_PATH"))
        self.assertFalse(hasattr(Security, "MDS_CDSAATTR_CDSAVERSION"))
        self.assertFalse(hasattr(Security, "MDS_CDSAATTR_VENDOR"))
        self.assertFalse(hasattr(Security, "MDS_CDSAATTR_DESC"))
        self.assertFalse(hasattr(Security, "MDS_CDSAATTR_INTERFACE_GUID"))
        self.assertFalse(hasattr(Security, "MDS_CDSAATTR_POLICY_STMT"))
        self.assertFalse(hasattr(Security, "MDS_CDSAATTR_EMMSPECVERSION"))
        self.assertFalse(hasattr(Security, "MDS_CDSAATTR_EMM_VERSION"))
        self.assertFalse(hasattr(Security, "MDS_CDSAATTR_EMM_VENDOR"))
        self.assertFalse(hasattr(Security, "MDS_CDSAATTR_EMM_TYPE"))
        self.assertFalse(hasattr(Security, "MDS_CDSAATTR_SSID"))
        self.assertFalse(hasattr(Security, "MDS_CDSAATTR_SERVICE_TYPE"))
        self.assertFalse(hasattr(Security, "MDS_CDSAATTR_NATIVE_SERVICES"))
        self.assertFalse(hasattr(Security, "MDS_CDSAATTR_DYNAMIC_FLAG"))
        self.assertFalse(hasattr(Security, "MDS_CDSAATTR_MULTITHREAD_FLAG"))
        self.assertFalse(hasattr(Security, "MDS_CDSAATTR_SERVICE_MASK"))
        self.assertFalse(hasattr(Security, "MDS_CDSAATTR_CSP_TYPE"))
        self.assertFalse(hasattr(Security, "MDS_CDSAATTR_CSP_FLAGS"))
        self.assertFalse(hasattr(Security, "MDS_CDSAATTR_CSP_CUSTOMFLAGS"))
        self.assertFalse(hasattr(Security, "MDS_CDSAATTR_USEE_TAGS"))
        self.assertFalse(hasattr(Security, "MDS_CDSAATTR_CONTEXT_TYPE"))
        self.assertFalse(hasattr(Security, "MDS_CDSAATTR_ALG_TYPE"))
        self.assertFalse(hasattr(Security, "MDS_CDSAATTR_GROUP_ID"))
        self.assertFalse(hasattr(Security, "MDS_CDSAATTR_ATTRIBUTE_TYPE"))
        self.assertFalse(hasattr(Security, "MDS_CDSAATTR_ATTRIBUTE_VALUE"))
        self.assertFalse(hasattr(Security, "MDS_CDSAATTR_PRODUCT_DESC"))
        self.assertFalse(hasattr(Security, "MDS_CDSAATTR_PRODUCT_VENDOR"))
        self.assertFalse(hasattr(Security, "MDS_CDSAATTR_PRODUCT_VERSION"))
        self.assertFalse(hasattr(Security, "MDS_CDSAATTR_PRODUCT_FLAGS"))
        self.assertFalse(hasattr(Security, "MDS_CDSAATTR_PRODUCT_CUSTOMFLAGS"))
        self.assertFalse(hasattr(Security, "MDS_CDSAATTR_STANDARD_DESC"))
        self.assertFalse(hasattr(Security, "MDS_CDSAATTR_STANDARD_VERSION"))
        self.assertFalse(hasattr(Security, "MDS_CDSAATTR_READER_DESC"))
        self.assertFalse(hasattr(Security, "MDS_CDSAATTR_READER_VENDOR"))
        self.assertFalse(hasattr(Security, "MDS_CDSAATTR_READER_VERSION"))
        self.assertFalse(hasattr(Security, "MDS_CDSAATTR_READER_FWVERSION"))
        self.assertFalse(hasattr(Security, "MDS_CDSAATTR_READER_FLAGS"))
        self.assertFalse(hasattr(Security, "MDS_CDSAATTR_READER_CUSTOMFLAGS"))
        self.assertFalse(hasattr(Security, "MDS_CDSAATTR_READER_SERIALNUMBER"))
        self.assertFalse(hasattr(Security, "MDS_CDSAATTR_SC_DESC"))
        self.assertFalse(hasattr(Security, "MDS_CDSAATTR_SC_VENDOR"))
        self.assertFalse(hasattr(Security, "MDS_CDSAATTR_SC_VERSION"))
        self.assertFalse(hasattr(Security, "MDS_CDSAATTR_SC_FWVERSION"))
        self.assertFalse(hasattr(Security, "MDS_CDSAATTR_SC_FLAGS"))
        self.assertFalse(hasattr(Security, "MDS_CDSAATTR_SC_CUSTOMFLAGS"))
        self.assertFalse(hasattr(Security, "MDS_CDSAATTR_SC_SERIALNUMBER"))
        self.assertFalse(hasattr(Security, "MDS_CDSAATTR_DL_TYPE"))
        self.assertFalse(hasattr(Security, "MDS_CDSAATTR_QUERY_LIMITS"))
        self.assertFalse(hasattr(Security, "MDS_CDSAATTR_CONJUNCTIVE_OPS"))
        self.assertFalse(hasattr(Security, "MDS_CDSAATTR_RELATIONAL_OPS"))
        self.assertFalse(hasattr(Security, "MDS_CDSAATTR_PROTOCOL"))
        self.assertFalse(hasattr(Security, "MDS_CDSAATTR_CERT_TYPEFORMAT"))
        self.assertFalse(hasattr(Security, "MDS_CDSAATTR_CRL_TYPEFORMAT"))
        self.assertFalse(hasattr(Security, "MDS_CDSAATTR_CERT_FIELDNAMES"))
        self.assertFalse(hasattr(Security, "MDS_CDSAATTR_BUNDLE_TYPEFORMAT"))
        self.assertFalse(hasattr(Security, "MDS_CDSAATTR_CERT_CLASSNAME"))
        self.assertFalse(hasattr(Security, "MDS_CDSAATTR_ROOTCERT"))
        self.assertFalse(hasattr(Security, "MDS_CDSAATTR_ROOTCERT_TYPEFORMAT"))
        self.assertFalse(hasattr(Security, "MDS_CDSAATTR_VALUE"))
        self.assertFalse(hasattr(Security, "MDS_CDSAATTR_REQCREDENTIALS"))
        self.assertFalse(hasattr(Security, "MDS_CDSAATTR_SAMPLETYPES"))
        self.assertFalse(hasattr(Security, "MDS_CDSAATTR_ACLSUBJECTTYPES"))
        self.assertFalse(hasattr(Security, "MDS_CDSAATTR_AUTHTAGS"))
        self.assertFalse(hasattr(Security, "MDS_CDSAATTR_USEETAG"))
        self.assertFalse(hasattr(Security, "MDS_CDSAATTR_RETRIEVALMODE"))
        self.assertFalse(hasattr(Security, "MDS_CDSAATTR_OID"))
        self.assertFalse(hasattr(Security, "MDS_CDSAATTR_XLATIONTYPEFORMAT"))
        self.assertFalse(hasattr(Security, "MDS_CDSAATTR_DEFAULT_TEMPLATE_TYPE"))
        self.assertFalse(hasattr(Security, "MDS_CDSAATTR_TEMPLATE_FIELD_NAMES"))
        self.assertFalse(hasattr(Security, "MDS_CDSAATTR_AUTHORITY_REQUEST_TYPE"))
        self.assertFalse(hasattr(Security, "MDS_OBJECT_NUM_RELATIONS"))
        self.assertFalse(hasattr(Security, "MDS_OBJECT_NUM_ATTRIBUTES"))
        self.assertFalse(hasattr(Security, "MDS_CDSADIR_NUM_RELATIONS"))
        self.assertFalse(hasattr(Security, "MDS_CDSADIR_CSSM_NUM_ATTRIBUTES"))
        self.assertFalse(hasattr(Security, "MDS_CDSADIR_EMM_NUM_ATTRIBUTES"))
        self.assertFalse(hasattr(Security, "MDS_CDSADIR_COMMON_NUM_ATTRIBUTES"))
        self.assertFalse(hasattr(Security, "MDS_CDSADIR_CSP_PRIMARY_NUM_ATTRIBUTES"))
        self.assertFalse(hasattr(Security, "MDS_CDSADIR_CSP_CAPABILITY_NUM_ATTRIBUTES"))
        self.assertFalse(
            hasattr(Security, "MDS_CDSADIR_CSP_ENCAPSULATED_PRODUCT_NUM_ATTRIBUTES")
        )
        self.assertFalse(hasattr(Security, "MDS_CDSADIR_CSP_SC_INFO_NUM_ATTRIBUTES"))
        self.assertFalse(hasattr(Security, "MDS_CDSADIR_DL_PRIMARY_NUM_ATTRIBUTES"))
        self.assertFalse(
            hasattr(Security, "MDS_CDSADIR_DL_ENCAPSULATED_PRODUCT_NUM_ATTRIBUTES")
        )
        self.assertFalse(hasattr(Security, "MDS_CDSADIR_CL_PRIMARY_NUM_ATTRIBUTES"))
        self.assertFalse(
            hasattr(Security, "MDS_CDSADIR_CL_ENCAPSULATED_PRODUCT_NUM_ATTRIBUTES")
        )
        self.assertFalse(hasattr(Security, "MDS_CDSADIR_TP_PRIMARY_NUM_ATTRIBUTES"))
        self.assertFalse(hasattr(Security, "MDS_CDSADIR_TP_OIDS_NUM_ATTRIBUTES"))
        self.assertFalse(
            hasattr(Security, "MDS_CDSADIR_TP_ENCAPSULATED_PRODUCT_NUM_ATTRIBUTES")
        )
        self.assertFalse(hasattr(Security, "MDS_CDSADIR_EMM_PRIMARY_NUM_ATTRIBUTES"))
        self.assertFalse(hasattr(Security, "MDS_CDSADIR_AC_PRIMARY_NUM_ATTRIBUTES"))
        self.assertFalse(
            hasattr(Security, "MDS_CDSADIR_SCHEMA_RELATONS_NUM_ATTRIBUTES")
        )
        self.assertFalse(
            hasattr(Security, "MDS_CDSADIR_SCHEMA_ATTRIBUTES_NUM_ATTRIBUTES")
        )
        self.assertFalse(hasattr(Security, "MDS_CDSADIR_SCHEMA_INDEXES_NUM_ATTRIBUTES"))



import Security
from PyObjCTools.TestSupport import TestCase, min_os_level

sec_protocol_key_update_complete_t = b"v"
sec_protocol_key_update_t = b"v@@?"
sec_protocol_challenge_complete_t = b"v@"
sec_protocol_challenge_t = b"v@@?"
sec_protocol_verify_complete_t = b"vB"
sec_protocol_verify_t = b"v@@@?"
sec_protocol_pre_shared_key_selection_t = b"v@@@?"


class TestSecProtocolOptions(TestCase):
    @min_os_level("10.14")
    def test_functions(self):
        Security.sec_protocol_options_set_local_identity
        Security.sec_protocol_options_add_tls_ciphersuite
        Security.sec_protocol_options_add_tls_ciphersuite_group
        Security.sec_protocol_options_set_tls_min_version
        Security.sec_protocol_options_set_tls_max_version
        Security.sec_protocol_options_add_tls_application_protocol
        Security.sec_protocol_options_set_tls_diffie_hellman_parameters

        self.assertArgHasType(
            Security.sec_protocol_options_set_tls_server_name, 1, b"n^t"
        )
        self.assertArgIsNullTerminated(
            Security.sec_protocol_options_set_tls_server_name, 1
        )

        Security.sec_protocol_options_add_pre_shared_key
        Security.sec_protocol_options_set_tls_tickets_enabled
        Security.sec_protocol_options_set_tls_is_fallback_attempt
        Security.sec_protocol_options_set_tls_resumption_enabled
        Security.sec_protocol_options_set_tls_false_start_enabled
        Security.sec_protocol_options_set_tls_ocsp_enabled
        Security.sec_protocol_options_set_tls_sct_enabled
        Security.sec_protocol_options_set_tls_renegotiation_enabled
        Security.sec_protocol_options_set_peer_authentication_required

    @min_os_level("10.15")
    def test_functions10_15(self):
        Security.sec_protocol_options_are_equal
        Security.sec_protocol_options_append_tls_ciphersuite
        Security.sec_protocol_options_append_tls_ciphersuite_group
        Security.sec_protocol_options_set_min_tls_protocol_version
        Security.sec_protocol_options_get_default_min_tls_protocol_version
        Security.sec_protocol_options_get_default_min_dtls_protocol_version
        Security.sec_protocol_options_set_max_tls_protocol_version
        Security.sec_protocol_options_get_default_max_tls_protocol_version
        Security.sec_protocol_options_get_default_max_dtls_protocol_version

        self.assertArgIsBlock(
            Security.sec_protocol_options_set_key_update_block,
            1,
            sec_protocol_key_update_t,
        )
        self.assertArgIsBlock(
            Security.sec_protocol_options_set_challenge_block,
            1,
            sec_protocol_challenge_t,
        )
        self.assertArgIsBlock(
            Security.sec_protocol_options_set_verify_block, 1, sec_protocol_verify_t
        )

        Security.sec_protocol_options_set_tls_pre_shared_key_identity_hint

        self.assertArgIsBlock(
            Security.sec_protocol_options_set_pre_shared_key_selection_block,
            1,
            sec_protocol_verify_t,
        )

    @min_os_level("10.16")
    def test_functions10_16(self):
        Security.sec_protocol_options_set_peer_authentication_optional



import Security
from PyObjCTools.TestSupport import TestCase, min_os_level, expectedFailure
import objc


class TestCode(TestCase):
    def test_types(self):
        self.assertIsCFType(Security.SecCodeRef)

    def test_constants(self):
        self.assertEqual(Security.kSecCSUseAllArchitectures, 1 << 0)

        self.assertIsInstance(Security.kSecGuestAttributeCanonical, str)
        self.assertIsInstance(Security.kSecGuestAttributeHash, str)
        self.assertIsInstance(Security.kSecGuestAttributeMachPort, str)
        self.assertIsInstance(Security.kSecGuestAttributePid, str)
        self.assertIsInstance(Security.kSecGuestAttributeDynamicCode, str)
        self.assertIsInstance(Security.kSecGuestAttributeDynamicCodeInfoPlist, str)

        self.assertEqual(Security.kSecCSInternalInformation, 1 << 0)
        self.assertEqual(Security.kSecCSSigningInformation, 1 << 1)
        self.assertEqual(Security.kSecCSRequirementInformation, 1 << 2)
        self.assertEqual(Security.kSecCSDynamicInformation, 1 << 3)
        self.assertEqual(Security.kSecCSContentInformation, 1 << 4)
        self.assertEqual(Security.kSecCSSkipResourceDirectory, 1 << 5)
        self.assertEqual(Security.kSecCSCalculateCMSDigest, 1 << 6)

        self.assertIsInstance(Security.kSecCodeInfoCertificates, str)
        self.assertIsInstance(Security.kSecCodeInfoChangedFiles, str)
        self.assertIsInstance(Security.kSecCodeInfoCMS, str)
        self.assertIsInstance(Security.kSecCodeInfoDesignatedRequirement, str)
        self.assertIsInstance(Security.kSecCodeInfoEntitlements, str)
        self.assertIsInstance(Security.kSecCodeInfoEntitlementsDict, str)
        self.assertIsInstance(Security.kSecCodeInfoFlags, str)
        self.assertIsInstance(Security.kSecCodeInfoFormat, str)
        self.assertIsInstance(Security.kSecCodeInfoDigestAlgorithm, str)
        self.assertIsInstance(Security.kSecCodeInfoIdentifier, str)
        self.assertIsInstance(Security.kSecCodeInfoImplicitDesignatedRequirement, str)
        self.assertIsInstance(Security.kSecCodeInfoMainExecutable, str)
        self.assertIsInstance(Security.kSecCodeInfoPList, str)
        self.assertIsInstance(Security.kSecCodeInfoRequirements, str)
        self.assertIsInstance(Security.kSecCodeInfoRequirementData, str)
        self.assertIsInstance(Security.kSecCodeInfoSource, str)
        self.assertIsInstance(Security.kSecCodeInfoStatus, str)
        self.assertIsInstance(Security.kSecCodeInfoTeamIdentifier, str)
        self.assertIsInstance(Security.kSecCodeInfoTime, str)
        self.assertIsInstance(Security.kSecCodeInfoTimestamp, str)
        self.assertIsInstance(Security.kSecCodeInfoTrust, str)
        self.assertIsInstance(Security.kSecCodeInfoUnique, str)

    @expectedFailure
    def test_constants_missing(self):
        self.assertIsInstance(Security.kSecGuestAttributeArchitecture, str)
        self.assertIsInstance(Security.kSecGuestAttributeSubarchitecture, str)
        self.assertIsInstance(Security.kSecCodeInfoPlatformIdentifier, str)

    @min_os_level("10.11.4")
    def test_constants_10_11_4(self):
        self.assertIsInstance(Security.kSecCodeInfoDigestAlgorithms, str)
        self.assertIsInstance(Security.kSecCodeInfoCdHashes, str)

    @min_os_level("10.12")
    def test_constants_10_12(self):
        self.assertIsInstance(Security.kSecGuestAttributeAudit, str)

    def test_functions(self):
        self.assertIsInstance(Security.SecCodeGetTypeID(), int)

        self.assertResultHasType(Security.SecCodeCopySelf, objc._C_INT)
        self.assertArgHasType(Security.SecCodeCopySelf, 0, objc._C_UINT)
        self.assertArgHasType(
            Security.SecCodeCopySelf, 1, objc._C_OUT + objc._C_PTR + objc._C_ID
        )
        self.assertArgIsCFRetained(Security.SecCodeCopySelf, 1)

        self.assertResultHasType(Security.SecCodeCopyStaticCode, objc._C_INT)
        self.assertArgHasType(Security.SecCodeCopyStaticCode, 0, objc._C_ID)
        self.assertArgHasType(Security.SecCodeCopyStaticCode, 1, objc._C_UINT)
        self.assertArgHasType(
            Security.SecCodeCopyStaticCode, 2, objc._C_OUT + objc._C_PTR + objc._C_ID
        )
        self.assertArgIsCFRetained(Security.SecCodeCopyStaticCode, 2)

        self.assertResultHasType(Security.SecCodeCopyHost, objc._C_INT)
        self.assertArgHasType(Security.SecCodeCopyHost, 0, objc._C_ID)
        self.assertArgHasType(Security.SecCodeCopyHost, 1, objc._C_UINT)
        self.assertArgHasType(
            Security.SecCodeCopyHost, 2, objc._C_OUT + objc._C_PTR + objc._C_ID
        )
        self.assertArgIsCFRetained(Security.SecCodeCopyHost, 2)

        self.assertResultHasType(Security.SecCodeCopyGuestWithAttributes, objc._C_INT)
        self.assertArgHasType(Security.SecCodeCopyGuestWithAttributes, 0, objc._C_ID)
        self.assertArgHasType(Security.SecCodeCopyGuestWithAttributes, 1, objc._C_ID)
        self.assertArgHasType(Security.SecCodeCopyGuestWithAttributes, 2, objc._C_UINT)
        self.assertArgHasType(
            Security.SecCodeCopyGuestWithAttributes,
            3,
            objc._C_OUT + objc._C_PTR + objc._C_ID,
        )
        self.assertArgIsCFRetained(Security.SecCodeCopyGuestWithAttributes, 3)

        self.assertResultHasType(Security.SecCodeCheckValidity, objc._C_INT)
        self.assertArgHasType(Security.SecCodeCheckValidity, 0, objc._C_ID)
        self.assertArgHasType(Security.SecCodeCheckValidity, 1, objc._C_UINT)
        self.assertArgHasType(Security.SecCodeCheckValidity, 2, objc._C_ID)

        self.assertResultHasType(Security.SecCodeCheckValidityWithErrors, objc._C_INT)
        self.assertArgHasType(Security.SecCodeCheckValidityWithErrors, 0, objc._C_ID)
        self.assertArgHasType(Security.SecCodeCheckValidityWithErrors, 1, objc._C_UINT)
        self.assertArgHasType(Security.SecCodeCheckValidityWithErrors, 2, objc._C_ID)
        self.assertArgHasType(
            Security.SecCodeCheckValidityWithErrors,
            3,
            objc._C_OUT + objc._C_PTR + objc._C_ID,
        )

        self.assertResultHasType(Security.SecCodeCopyPath, objc._C_INT)
        self.assertArgHasType(Security.SecCodeCopyPath, 0, objc._C_ID)
        self.assertArgHasType(Security.SecCodeCopyPath, 1, objc._C_UINT)
        self.assertArgHasType(
            Security.SecCodeCopyPath, 2, objc._C_OUT + objc._C_PTR + objc._C_ID
        )
        self.assertArgIsCFRetained(Security.SecCodeCopyPath, 2)

        self.assertResultHasType(Security.SecCodeCopyDesignatedRequirement, objc._C_INT)
        self.assertArgHasType(Security.SecCodeCopyDesignatedRequirement, 0, objc._C_ID)
        self.assertArgHasType(
            Security.SecCodeCopyDesignatedRequirement, 1, objc._C_UINT
        )
        self.assertArgHasType(
            Security.SecCodeCopyDesignatedRequirement,
            2,
            objc._C_OUT + objc._C_PTR + objc._C_ID,
        )
        self.assertArgIsCFRetained(Security.SecCodeCopyDesignatedRequirement, 2)

        self.assertResultHasType(Security.SecCodeCopySigningInformation, objc._C_INT)
        self.assertArgHasType(Security.SecCodeCopySigningInformation, 0, objc._C_ID)
        self.assertArgHasType(Security.SecCodeCopySigningInformation, 1, objc._C_UINT)
        self.assertArgHasType(
            Security.SecCodeCopySigningInformation,
            2,
            objc._C_OUT + objc._C_PTR + objc._C_ID,
        )
        self.assertArgIsCFRetained(Security.SecCodeCopySigningInformation, 2)

        self.assertResultHasType(Security.SecCodeMapMemory, objc._C_INT)
        self.assertArgHasType(Security.SecCodeMapMemory, 0, objc._C_ID)
        self.assertArgHasType(Security.SecCodeMapMemory, 1, objc._C_UINT)

        self.assertArgIsOut(Security.SecCodeCreateWithXPCMessage, 2)
        self.assertArgIsCFRetained(Security.SecCodeCreateWithXPCMessage, 2)



import Security
from PyObjCTools.TestSupport import TestCase


class Testcertextensions(TestCase):
    def test_unsuppported(self):
        self.assertFalse(hasattr(Security, "GNT_OtherNamed"))
        self.assertFalse(hasattr(Security, "GNT_RFC822Named"))
        self.assertFalse(hasattr(Security, "GNT_DNSNamed"))
        self.assertFalse(hasattr(Security, "GNT_X400Addressd"))
        self.assertFalse(hasattr(Security, "GNT_DirectoryNamed"))
        self.assertFalse(hasattr(Security, "GNT_EdiPartyNamed"))
        self.assertFalse(hasattr(Security, "GNT_URId"))
        self.assertFalse(hasattr(Security, "GNT_IPAddressd"))
        self.assertFalse(hasattr(Security, "GNT_RegisteredID"))
        self.assertFalse(hasattr(Security, "CE_OtherName"))
        self.assertFalse(hasattr(Security, "CE_GeneralName"))
        self.assertFalse(hasattr(Security, "CE_GeneralNames"))
        self.assertFalse(hasattr(Security, "CE_AuthorityKeyID"))
        self.assertFalse(hasattr(Security, "CE_KU_DigitalSignature"))
        self.assertFalse(hasattr(Security, "CE_KU_NonRepudiation"))
        self.assertFalse(hasattr(Security, "CE_KU_KeyEncipherment"))
        self.assertFalse(hasattr(Security, "CE_KU_DataEncipherment"))
        self.assertFalse(hasattr(Security, "CE_KU_KeyAgreement"))
        self.assertFalse(hasattr(Security, "CE_KU_KeyCertSign"))
        self.assertFalse(hasattr(Security, "CE_KU_CRLSign"))
        self.assertFalse(hasattr(Security, "CE_KU_EncipherOnly"))
        self.assertFalse(hasattr(Security, "CE_KU_DecipherOnly"))
        self.assertFalse(hasattr(Security, "CE_CR_Unspecified"))
        self.assertFalse(hasattr(Security, "CE_CR_KeyCompromise"))
        self.assertFalse(hasattr(Security, "CE_CR_CACompromise"))
        self.assertFalse(hasattr(Security, "CE_CR_AffiliationChanged"))
        self.assertFalse(hasattr(Security, "CE_CR_Superseded"))
        self.assertFalse(hasattr(Security, "CE_CR_CessationOfOperation"))
        self.assertFalse(hasattr(Security, "CE_CR_CertificateHold"))
        self.assertFalse(hasattr(Security, "CE_CR_RemoveFromCRL"))
        self.assertFalse(hasattr(Security, "CE_ExtendedKeyUsage"))
        self.assertFalse(hasattr(Security, "CE_BasicConstraints"))
        self.assertFalse(hasattr(Security, "SecCEBasicConstraints"))
        self.assertFalse(hasattr(Security, "CE_PolicyQualifierInfo"))
        self.assertFalse(hasattr(Security, "CE_PolicyInformation"))
        self.assertFalse(hasattr(Security, "CE_CertPolicies"))
        self.assertFalse(hasattr(Security, "CE_CD_Unspecified"))
        self.assertFalse(hasattr(Security, "CE_CD_KeyCompromise"))
        self.assertFalse(hasattr(Security, "CE_CD_CACompromise"))
        self.assertFalse(hasattr(Security, "CE_CD_AffiliationChanged"))
        self.assertFalse(hasattr(Security, "CE_CD_Superseded"))
        self.assertFalse(hasattr(Security, "CE_CD_CessationOfOperation"))
        self.assertFalse(hasattr(Security, "CE_CD_CertificateHold"))
        self.assertFalse(hasattr(Security, "CE_CDNT_FullNamed"))
        self.assertFalse(hasattr(Security, "CE_CDNT_NameRelativeToCrlIssuer"))
        self.assertFalse(hasattr(Security, "CE_DistributionPointName"))
        self.assertFalse(hasattr(Security, "CE_CRLDistributionPoint"))
        self.assertFalse(hasattr(Security, "CE_CRLDistPointsSyntax"))
        self.assertFalse(hasattr(Security, "CE_AccessDescription"))
        self.assertFalse(hasattr(Security, "CE_AuthorityInfoAccess"))
        self.assertFalse(hasattr(Security, "CE_SemanticsInformation"))
        self.assertFalse(hasattr(Security, "CE_QC_Statement"))
        self.assertFalse(hasattr(Security, "CE_QC_Statements"))
        self.assertFalse(hasattr(Security, "CE_IssuingDistributionPoint"))
        self.assertFalse(hasattr(Security, "CE_GeneralSubtree"))
        self.assertFalse(hasattr(Security, "CE_GeneralSubtrees"))
        self.assertFalse(hasattr(Security, "CE_NameConstraints"))
        self.assertFalse(hasattr(Security, "CE_PolicyMapping"))
        self.assertFalse(hasattr(Security, "CE_PolicyMappings"))
        self.assertFalse(hasattr(Security, "CE_PolicyConstraints"))
        self.assertFalse(hasattr(Security, "DT_AuthorityKeyIDd"))
        self.assertFalse(hasattr(Security, "DT_SubjectKeyIDd"))
        self.assertFalse(hasattr(Security, "DT_KeyUsaged"))
        self.assertFalse(hasattr(Security, "DT_SubjectAltNamed"))
        self.assertFalse(hasattr(Security, "DT_IssuerAltNamed"))
        self.assertFalse(hasattr(Security, "DT_ExtendedKeyUsaged"))
        self.assertFalse(hasattr(Security, "DT_BasicConstraintsd"))
        self.assertFalse(hasattr(Security, "DT_CertPoliciesd"))
        self.assertFalse(hasattr(Security, "DT_NetscapeCertTyped"))
        self.assertFalse(hasattr(Security, "DT_CrlNumberd"))
        self.assertFalse(hasattr(Security, "DT_DeltaCrld"))
        self.assertFalse(hasattr(Security, "DT_CrlReasond"))
        self.assertFalse(hasattr(Security, "DT_CrlDistributionPointsd"))
        self.assertFalse(hasattr(Security, "DT_IssuingDistributionPointd"))
        self.assertFalse(hasattr(Security, "DT_AuthorityInfoAccessd"))
        self.assertFalse(hasattr(Security, "DT_Otherd"))
        self.assertFalse(hasattr(Security, "DT_QC_Statementsd"))
        self.assertFalse(hasattr(Security, "DT_NameConstraintsd"))
        self.assertFalse(hasattr(Security, "DT_PolicyMappingsd"))
        self.assertFalse(hasattr(Security, "DT_PolicyConstraintsd"))
        self.assertFalse(hasattr(Security, "DT_InhibitAnyPolicy"))
        self.assertFalse(hasattr(Security, "CE_Data"))
        self.assertFalse(hasattr(Security, "CE_DataAndType"))



import Security
from PyObjCTools.TestSupport import TestCase, min_os_level


class TestSecAccess(TestCase):
    @min_os_level("10.16")
    def test_constants10_16(self):
        self.assertIsInstance(Security.kSecSharedPassword, str)

    @min_os_level("10.16")
    def test_functions10_16(self):
        # XXX: This crashes the bridge
        self.assertArgIsBlock(Security.SecAddSharedWebCredential, 3, b"v@")

        self.assertArgIsBlock(Security.SecRequestSharedWebCredential, 2, b"v@@")

        Security.SecCreateSharedWebCredentialPassword



import Security
from PyObjCTools.TestSupport import TestCase, min_os_level
import objc


class TestCMSEncoder(TestCase):
    def test_types(self):
        self.assertIsCFType(Security.CMSEncoderRef)

    def test_constants(self):

        self.assertEqual(Security.kCMSAttrNone, 0x0000)
        self.assertEqual(Security.kCMSAttrSmimeCapabilities, 0x0001)
        self.assertEqual(Security.kCMSAttrSmimeEncryptionKeyPrefs, 0x0002)
        self.assertEqual(Security.kCMSAttrSmimeMSEncryptionKeyPrefs, 0x0004)
        self.assertEqual(Security.kCMSAttrSigningTime, 0x0008)
        self.assertEqual(Security.kCMSAttrAppleCodesigningHashAgility, 0x0010)
        self.assertEqual(Security.kCMSAttrAppleCodesigningHashAgilityV2, 0x0020)
        self.assertEqual(Security.kCMSAttrAppleExpirationTime, 0x0040)

        self.assertEqual(Security.kCMSCertificateNone, 0)
        self.assertEqual(Security.kCMSCertificateSignerOnly, 1)
        self.assertEqual(Security.kCMSCertificateChain, 2)
        self.assertEqual(Security.kCMSCertificateChainWithRoot, 3)
        self.assertEqual(Security.kCMSCertificateChainWithRootOrFail, 4)

    @min_os_level("10.11")
    def test_constants_10_11(self):
        self.assertIsInstance(Security.kCMSEncoderDigestAlgorithmSHA1, str)
        self.assertIsInstance(Security.kCMSEncoderDigestAlgorithmSHA256, str)

    def test_functions(self):
        self.assertIsInstance(Security.CMSEncoderGetTypeID(), int)

        self.assertResultHasType(Security.CMSEncoderCreate, objc._C_INT)
        self.assertArgHasType(
            Security.CMSEncoderCreate, 0, objc._C_OUT + objc._C_PTR + objc._C_ID
        )
        self.assertArgIsCFRetained(Security.CMSEncoderCreate, 0)

        self.assertResultHasType(Security.CMSEncoderAddSigners, objc._C_INT)
        self.assertArgHasType(Security.CMSEncoderAddSigners, 0, objc._C_ID)
        self.assertArgHasType(Security.CMSEncoderAddSigners, 1, objc._C_ID)

        self.assertResultHasType(Security.CMSEncoderCopySigners, objc._C_INT)
        self.assertArgHasType(Security.CMSEncoderCopySigners, 0, objc._C_ID)
        self.assertArgHasType(
            Security.CMSEncoderCopySigners, 1, objc._C_OUT + objc._C_PTR + objc._C_ID
        )
        self.assertArgIsCFRetained(Security.CMSEncoderCopySigners, 1)

        self.assertResultHasType(Security.CMSEncoderAddRecipients, objc._C_INT)
        self.assertArgHasType(Security.CMSEncoderAddRecipients, 0, objc._C_ID)
        self.assertArgHasType(Security.CMSEncoderAddRecipients, 1, objc._C_ID)

        self.assertResultHasType(Security.CMSEncoderCopyRecipients, objc._C_INT)
        self.assertArgHasType(Security.CMSEncoderCopyRecipients, 0, objc._C_ID)
        self.assertArgHasType(
            Security.CMSEncoderCopyRecipients, 1, objc._C_OUT + objc._C_PTR + objc._C_ID
        )
        self.assertArgIsCFRetained(Security.CMSEncoderCopyRecipients, 1)

        self.assertResultHasType(Security.CMSEncoderSetHasDetachedContent, objc._C_INT)
        self.assertArgHasType(Security.CMSEncoderSetHasDetachedContent, 0, objc._C_ID)
        self.assertArgHasType(
            Security.CMSEncoderSetHasDetachedContent, 1, objc._C_NSBOOL
        )

        self.assertResultHasType(Security.CMSEncoderGetHasDetachedContent, objc._C_INT)
        self.assertArgHasType(Security.CMSEncoderGetHasDetachedContent, 0, objc._C_ID)
        self.assertArgHasType(
            Security.CMSEncoderGetHasDetachedContent,
            1,
            objc._C_OUT + objc._C_PTR + objc._C_NSBOOL,
        )

        self.assertFalse(hasattr(Security, "CMSEncoderSetEncapsulatedContentType"))

        self.assertResultHasType(
            Security.CMSEncoderSetEncapsulatedContentTypeOID, objc._C_INT
        )
        self.assertArgHasType(
            Security.CMSEncoderSetEncapsulatedContentTypeOID, 0, objc._C_ID
        )
        self.assertArgHasType(
            Security.CMSEncoderSetEncapsulatedContentTypeOID, 1, objc._C_ID
        )

        self.assertResultHasType(
            Security.CMSEncoderCopyEncapsulatedContentType, objc._C_INT
        )
        self.assertArgHasType(
            Security.CMSEncoderCopyEncapsulatedContentType, 0, objc._C_ID
        )
        self.assertArgHasType(
            Security.CMSEncoderCopyEncapsulatedContentType,
            1,
            objc._C_OUT + objc._C_PTR + objc._C_ID,
        )
        self.assertArgIsCFRetained(Security.CMSEncoderCopyEncapsulatedContentType, 1)

        self.assertResultHasType(Security.CMSEncoderAddSupportingCerts, objc._C_INT)
        self.assertArgHasType(Security.CMSEncoderAddSupportingCerts, 0, objc._C_ID)
        self.assertArgHasType(Security.CMSEncoderAddSupportingCerts, 1, objc._C_ID)

        self.assertResultHasType(Security.CMSEncoderCopySupportingCerts, objc._C_INT)
        self.assertArgHasType(Security.CMSEncoderCopySupportingCerts, 0, objc._C_ID)
        self.assertArgHasType(
            Security.CMSEncoderCopySupportingCerts,
            1,
            objc._C_OUT + objc._C_PTR + objc._C_ID,
        )
        self.assertArgIsCFRetained(Security.CMSEncoderCopySupportingCerts, 1)

        self.assertResultHasType(Security.CMSEncoderAddSignedAttributes, objc._C_INT)
        self.assertArgHasType(Security.CMSEncoderAddSignedAttributes, 0, objc._C_ID)
        self.assertArgHasType(Security.CMSEncoderAddSignedAttributes, 1, objc._C_UINT)

        self.assertResultHasType(
            Security.CMSEncoderSetCertificateChainMode, objc._C_INT
        )
        self.assertArgHasType(Security.CMSEncoderSetCertificateChainMode, 0, objc._C_ID)
        self.assertArgHasType(
            Security.CMSEncoderSetCertificateChainMode, 1, objc._C_UINT
        )

        self.assertResultHasType(
            Security.CMSEncoderGetCertificateChainMode, objc._C_INT
        )
        self.assertArgHasType(Security.CMSEncoderGetCertificateChainMode, 0, objc._C_ID)
        self.assertArgHasType(
            Security.CMSEncoderGetCertificateChainMode,
            1,
            objc._C_OUT + objc._C_PTR + objc._C_UINT,
        )

        self.assertResultHasType(Security.CMSEncoderUpdateContent, objc._C_INT)
        self.assertArgHasType(Security.CMSEncoderUpdateContent, 0, objc._C_ID)
        self.assertArgHasType(
            Security.CMSEncoderUpdateContent, 1, objc._C_IN + objc._C_PTR + objc._C_VOID
        )
        self.assertArgSizeInArg(Security.CMSEncoderUpdateContent, 1, 2)
        self.assertArgHasType(Security.CMSEncoderUpdateContent, 2, objc._C_ULNG)

        self.assertResultHasType(Security.CMSEncoderCopyEncodedContent, objc._C_INT)
        self.assertArgHasType(Security.CMSEncoderCopyEncodedContent, 0, objc._C_ID)
        self.assertArgHasType(
            Security.CMSEncoderCopyEncodedContent,
            1,
            objc._C_OUT + objc._C_PTR + objc._C_ID,
        )
        self.assertArgIsCFRetained(Security.CMSEncoderCopyEncodedContent, 1)

        self.assertFalse(hasattr(Security, "CMSEncode"))

        self.assertResultHasType(Security.CMSEncodeContent, objc._C_INT)
        self.assertArgHasType(Security.CMSEncodeContent, 0, objc._C_ID)
        self.assertArgHasType(Security.CMSEncodeContent, 1, objc._C_ID)
        self.assertArgHasType(Security.CMSEncodeContent, 2, objc._C_ID)
        self.assertArgHasType(Security.CMSEncodeContent, 3, objc._C_NSBOOL)
        self.assertArgHasType(Security.CMSEncodeContent, 4, objc._C_UINT)
        self.assertArgHasType(
            Security.CMSEncodeContent, 5, objc._C_IN + objc._C_PTR + objc._C_VOID
        )
        self.assertArgSizeInArg(Security.CMSEncodeContent, 5, 6)
        self.assertArgHasType(Security.CMSEncodeContent, 6, objc._C_ULNG)
        self.assertArgHasType(
            Security.CMSEncodeContent, 7, objc._C_OUT + objc._C_PTR + objc._C_ID
        )
        self.assertArgIsCFRetained(Security.CMSEncodeContent, 7)

        self.assertResultHasType(Security.CMSEncoderCopySignerTimestamp, objc._C_INT)
        self.assertArgHasType(Security.CMSEncoderCopySignerTimestamp, 0, objc._C_ID)
        self.assertArgHasType(Security.CMSEncoderCopySignerTimestamp, 1, objc._C_ULNG)
        self.assertArgHasType(
            Security.CMSEncoderCopySignerTimestamp,
            2,
            objc._C_OUT + objc._C_PTR + objc._C_DBL,
        )

    @min_os_level("10.10")
    def test_functions_10_10(self):
        self.assertResultHasType(
            Security.CMSEncoderCopySignerTimestampWithPolicy, objc._C_INT
        )
        self.assertArgHasType(
            Security.CMSEncoderCopySignerTimestampWithPolicy, 0, objc._C_ID
        )
        self.assertArgHasType(
            Security.CMSEncoderCopySignerTimestampWithPolicy, 1, objc._C_ID
        )
        self.assertArgHasType(
            Security.CMSEncoderCopySignerTimestampWithPolicy, 2, objc._C_ULNG
        )
        self.assertArgHasType(
            Security.CMSEncoderCopySignerTimestampWithPolicy,
            3,
            objc._C_OUT + objc._C_PTR + objc._C_DBL,
        )

    @min_os_level("10.11")
    def test_functions_10_11(self):
        self.assertResultHasType(Security.CMSEncoderSetSignerAlgorithm, objc._C_INT)
        self.assertArgHasType(Security.CMSEncoderSetSignerAlgorithm, 0, objc._C_ID)
        self.assertArgHasType(Security.CMSEncoderSetSignerAlgorithm, 1, objc._C_ID)



import Security
from PyObjCTools.TestSupport import TestCase


class Testcssmkrapi(TestCase):
    def test_unsuppported(self):
        self.assertFalse(hasattr(Security, "CSSM_KR_NAME"))
        self.assertFalse(hasattr(Security, "CSSM_KR_PROFILE"))
        self.assertFalse(hasattr(Security, "CSSM_KR_WRAPPEDPRODUCT_INFO"))
        self.assertFalse(hasattr(Security, "CSSM_KRSUBSERVICE"))
        self.assertFalse(hasattr(Security, "CSSM_KR_INDIV_POLICY"))
        self.assertFalse(hasattr(Security, "CSSM_KR_ENT_POLICY"))
        self.assertFalse(hasattr(Security, "CSSM_KR_LE_MAN_POLICY"))
        self.assertFalse(hasattr(Security, "CSSM_KR_LE_USE_POLICY"))
        self.assertFalse(hasattr(Security, "CSSM_KR_INDIV"))
        self.assertFalse(hasattr(Security, "CSSM_KR_ENT"))
        self.assertFalse(hasattr(Security, "CSSM_KR_LE_MAN"))
        self.assertFalse(hasattr(Security, "CSSM_KR_LE_USE"))
        self.assertFalse(hasattr(Security, "CSSM_KR_LE"))
        self.assertFalse(hasattr(Security, "CSSM_KR_OPTIMIZE"))
        self.assertFalse(hasattr(Security, "CSSM_KR_DROP_WORKFACTOR"))
        self.assertFalse(hasattr(Security, "CSSM_KR_POLICY_LIST_ITEM"))
        self.assertFalse(hasattr(Security, "CSSM_KR_POLICY_INFO"))
        self.assertFalse(hasattr(Security, "CSSM_KR_SetEnterpriseRecoveryPolicy"))
        self.assertFalse(hasattr(Security, "CSSM_KR_CreateRecoveryRegistrationContext"))
        self.assertFalse(hasattr(Security, "CSSM_KR_CreateRecoveryEnablementContext"))
        self.assertFalse(hasattr(Security, "CSSM_KR_CreateRecoveryRequestContext"))
        self.assertFalse(hasattr(Security, "CSSM_KR_GetPolicyInfo"))
        self.assertFalse(hasattr(Security, "CSSM_KR_RegistrationRequest"))
        self.assertFalse(hasattr(Security, "CSSM_KR_RegistrationRetrieve"))
        self.assertFalse(hasattr(Security, "CSSM_KR_GenerateRecoveryFields"))
        self.assertFalse(hasattr(Security, "CSSM_KR_ProcessRecoveryFields"))
        self.assertFalse(hasattr(Security, "CSSM_KR_RecoveryRequest"))
        self.assertFalse(hasattr(Security, "CSSM_KR_RecoveryRetrieve"))
        self.assertFalse(hasattr(Security, "CSSM_KR_GetRecoveredObject"))
        self.assertFalse(hasattr(Security, "CSSM_KR_RecoveryRequestAbort"))
        self.assertFalse(hasattr(Security, "CSSM_KR_QueryPolicyInfo"))
        self.assertFalse(hasattr(Security, "CSSM_KR_PassThrough"))
